
  /**
  ***  determine the board type and emulate legacy
  ***  call if needed
  **/

  tpro_boolean_t bIsTsync;
  CHECK(TPRO_boardIsTsync(hnd, &bIsTsync),
        TPRO_SUCCESS);
  
  if(bIsTsync == TPRO_TRUE)
  {
    TSYNC_TableTypeObj pObj;
    pObj.type = RS_TT_CURRENT;
    TSYNC_ReferenceTableObj pObj2;
    TSYNC_ERROR err = TSYNC_RS_getTable(hnd, &pObj, &pObj2);
    if (err != TSYNC_SUCCESS)
    {
        printf(" *** Error: %s.\n", tsync_strerror(err));
        return TPRO_COMM_ERR;
    }
    
    int i;
    for(i = 0; i < TSYNC_TABLE_ENTRY_NUM; i ++)
    {
        const char SELF_STRING[] = "self";

        uint32_t alreadyEnabled; 
        unsigned char enabling = *ENABLED_NAME;
        unsigned char eligible ;

        alreadyEnabled = pObj2.rows[i].enab;

        /* Assume we shouldn't do an enable/disable transaction for
         * this reference until we determine otherwise. */
        eligible = 0;

        /* If we're enabling, enable any references which are disabled
         * and do not have the ASCII string "self" as both
         * references */
        if ((enabling == 1) && 
            (!alreadyEnabled) &&
            ((strcmp(pObj2.rows[i].time,  SELF_STRING) != 0) ||
             (strcmp(pObj2.rows[i].pps,  SELF_STRING) != 0))) {
            eligible = 1;
        }

        /* If we're disabling, disable any active references. */
        if ((enabling == 0) && (alreadyEnabled)) {
            eligible = 1;
        }

        /* Skip empty entries. */
        if (pObj2.rows[i].time[0] == '\0' && pObj2.rows[i].pps[0] == '\0') {
            eligible = 0;
        }
        
        if (eligible)
        {
            //printf(" Set enabled: %u\n", enabling);
            err = TSYNC_RS_setEnable(hnd, i, enabling);
            if (err != TSYNC_SUCCESS)
            {
                printf(" *** Error: %s.\n", tsync_strerror(err));
                return TPRO_COMM_ERR;
            }
        }
    }
        
    return (TPRO_SUCCESS);
  }
