#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[32];
    char            *pgmname = argv[0];
    TSYNC_ERROR     err = TSYNC_SUCCESS;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }
    
    //Get IRIG Output Signature Control
    {
        IO_PORT inst = (IO_PORT)0;
        SIG_CTL sig;
        
        printf(" Get IRIG Output Signature Control\n");
        err = TSYNC_IP_getSigCtrl(hnd, inst, &sig);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  IRIG Output Signature Control: %u\n", sig);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set IRIG Output Signature Control
    {
        IO_PORT inst = (IO_PORT)0;
        SIG_CTL sig = SIG_CTL_SYNC;
        
        printf(" Set IRIG Output Signature Control (%u)\n", sig);
        err = TSYNC_IP_setSigCtrl(hnd, inst, sig);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get IRIG Output Offset
    {
        IO_PORT inst = (IO_PORT)0;
        int nOffset;
        
        printf(" Get IRIG Output Offset\n");
        err = TSYNC_IP_getOffset(hnd, inst, &nOffset);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  IRIG Output Offset: %u\n", nOffset);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set IRIG Output Offset
    {
        IO_PORT inst = (IO_PORT)0;
        int nOffset = 0;
        
        printf(" Set IRIG Output Offset (%u)\n", nOffset);
        err = TSYNC_IP_setOffset(hnd, inst, nOffset);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set IRIG Output Local
    /*{
        TSYNC_LocalClockObj pObj;
        pObj. = //set some values!
        
        printf(" Set IRIG Output Local\n");
        err = TSYNC_IP_setLocal(hnd, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }*/
    
    //Get IRIG Output Local
    {
        IO_PORT inst = (IO_PORT)0;
        TSYNC_LocalClockObj pObj;
        
        printf(" Get IRIG Output Local\n");
        err = TSYNC_IP_getLocal(hnd, inst, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf(    "  IRIG Output Local:\n"
                    "   Rule: %u\n"
                    "   In month: %u\n"
                    "   In wom: %u\n"
                    "   In dow: %u\n"
                    "   In hour: %u\n"
                    "   Out month: %u\n"
                    "   Out wom: %u\n"
                    "   Out dow: %u\n"
                    "   Out hour: %u\n"
                    "   Offset: %u\n"
                    "   Tz: %u\n",
                    pObj.rule.ref,
                    pObj.rule.in.month,
                    pObj.rule.in.wom,
                    pObj.rule.in.dow,
                    pObj.rule.in.hour,
                    pObj.rule.out.month,
                    pObj.rule.out.wom,
                    pObj.rule.out.dow,
                    pObj.rule.out.hour,
                    pObj.rule.offset,
                    pObj.tz);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    
    
    
    
    
    //Set IRIG Amplitude
    {
        IO_PORT inst = (IO_PORT)0;
        unsigned int amp = 0;
        
        printf(" Set IRIG Amplitude (%u)\n", amp);
        err = TSYNC_IP_setAmplitude(hnd, inst, amp);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get IRIG Amplitude
    {
        IO_PORT inst = (IO_PORT)0;
        unsigned int amp;
        
        printf(" Get IRIG Amplitude\n");
        err = TSYNC_IP_getAmplitude(hnd, inst, &amp);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  IRIG Amplitude: %u\n", amp);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set IRIG Format
    {
        IO_PORT inst = (IO_PORT)0;
        IL_FMT format = IL_FMT_IRIG_A;
        
        printf(" Set IRIG Format (%u)\n", format);
        err = TSYNC_IP_setFormat(hnd, inst, format);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set IRIG Frequency
    {
        IO_PORT inst = (IO_PORT)0;
        IL_FRQ freq = IL_FRQ_100HZ;
        
        printf(" Set IRIG Frequency (%u)\n", freq);
        err = TSYNC_IP_setFreq(hnd, inst, freq);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set IRIG Coded Expression
    {
        IO_PORT inst = (IO_PORT)0;
        IL_CE ce = IL_CE_BCDT_CF_SBS;
        
        printf(" Set IRIG Coded Expression (%u)\n", ce);
        err = TSYNC_IP_setCodedExpr(hnd, inst, ce);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set IRIG Control Field
    {
        IO_PORT inst = (IO_PORT)0;
        IL_CF cf = IL_CF_200_04;
        
        printf(" Set IRIG Control Field (%u)\n", cf);
        err = TSYNC_IP_setCtrlField(hnd, inst, cf);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get IRIG Format
    {
        IO_PORT inst = (IO_PORT)0;
        IL_FMT format;
        
        printf(" Get IRIG Format\n");
        err = TSYNC_IP_getFormat(hnd, inst, &format);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  IRIG Format: %u\n", format);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get IRIG Modulation
    {
        IO_PORT inst = (IO_PORT)0;
        IL_MOD mod;
        
        printf(" Get IRIG Modulation\n");
        err = TSYNC_IP_getMod(hnd, inst, &mod);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  IRIG Modulation: %u\n", mod);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get IRIG Frequency
    {
        IO_PORT inst = (IO_PORT)0;
        IL_FRQ freq;
        
        printf(" Get IRIG Frequency\n");
        err = TSYNC_IP_getFreq(hnd, inst, &freq);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  IRIG Frequency: %u\n", freq);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get IRIG Coded Expression
    {
        IO_PORT inst = (IO_PORT)0;
        IL_CE ce;
        
        printf(" Get IRIG Coded Expression\n");
        err = TSYNC_IP_getCodedExpr(hnd, inst, &ce);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  IRIG Coded Expression: %u\n", ce);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get IRIG Control Field
    {
        IO_PORT inst = (IO_PORT)0;
        IL_CF cf;
        
        printf(" Get IRIG Control Field\n");
        err = TSYNC_IP_getCtrlField(hnd, inst, &cf);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  IRIG Control Field: %u\n", cf);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set IRIG Message
    /*{
        TSYNC_IRIGMessageObj pObj;
        int i;
        for(i = 0; i < TSYNC_IR_SUBFRAME_NUM; i++)
        {
            pObj.subframes[i] = i;
        }
        
        printf(" Set IRIG Message\n");
        err = TSYNC_IP_setMessage(hnd, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }*/
    
        
    /*
     * close the handle to the board
     */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n", DEFAULT_DEV);
    (void)printf("\t-t   image type (optional, 0 - %u)\n", UL_IMG_NUM_ITEMS - 1);
    (void)printf("\n");
}
