#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:"

/********************************************************
**
** Test Module: GetStatus.c
**
** Purpose:     Get the current upgrade status from the
**              board.
**
**
*********************************************************/

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[32];
    char            *pgmname = argv[0];
    TSYNC_ERROR     err = TSYNC_SUCCESS;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }
    
    /*
     * construct the device name
     */
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    //Get init status
    {
        printf(" Initializer Status Results\n");
        
        int bMorePages = 1;
        int nCurPage = 0;
                    printf( "\tIndex\t"
                            "Result\t"
                            "Module\n");
        while(bMorePages)
        {
            TSYNC_InitStatusResult pObj;
            err = TSYNC_IN_getStatus(hnd, nCurPage, &pObj);
            
            if (err == TSYNC_SUCCESS)
            {
                bMorePages = (pObj.more != 0);
                
                //printf(" Page Num: %u\n", pObj.pageNum);
                //printf(" More? %u\n", pObj.more);
                
                int i;
                for(i = 0; i < TSYNC_INIT_RESULT_NUM; i++)
                {
                    //if we hit an invalid cap item number, no more caps to parse
                    if(pObj.results[i].result == RC_UNINITIALIZED)
                    {
                        break;
                    }
                    printf( "\t%03u\t"
                            "%02u\t"
                            "%s",
                            nCurPage * TSYNC_INIT_RESULT_NUM + i,
                            pObj.results[i].result,
                            pObj.results[i].module);
                    
                    if(pObj.results[i].result != RC_NO_ERROR &&
                        pObj.results[i].result != RC_UNINITIALIZED)
                    {
                        printf( "\t(%s)",
                                tsync_strerror(pObj.results[i].result));
                    }
                            
                    printf( "\n");
                }
            }
            else
            {
                printf("  Error: %s.\n", tsync_strerror(err));
                break;
            }
            
            nCurPage++;
        }
    }
    
    /*
     * close the handle to the board
     */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n", DEFAULT_DEV);
    (void)printf("\n");
}
