
#include "example.h"

int main (int argc, char *argv[])
{
    char            *pgmname = argv[0];
    char           fullDevName[32];
    int            tmp;
    TSYNC_BoardHandle   hnd;
    int             rc;
    int             ret = 0;
    TSYNC_ERROR     err = TSYNC_SUCCESS;

    /* If invalid number of arguments... */
    if(argc != 8 && argc != 4 && argc != 9)
    {
        printf( " Usage:\n"
                "  SetTime <device index> <years> <doy> <hours> <minutes> <seconds> <nanoseconds>\n"
                "  SetTime <device index> <seconds> <nanoseconds>\n"
                "  SetTime <device index> <years> <doy> <hours> <minutes> <seconds> <ms> <us>\n");
               return 1;
    }
    
    /* Construct the TSYNC/TSAT device name */
    sprintf(fullDevName, "%s%s%d", DEVICE_ROOT, DEFAULT_DEV, atoi(argv[1]));
    
    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }
    
    //Set Time DOY
    if (argc == 8)
    {
        TSYNC_TimeObj   tsyncTime = {0};
        /*
        **  read into time object
        */
        sscanf (argv[2], "%10d", &tmp);
        tsyncTime.years = (unsigned int) tmp;
    
        sscanf (argv[3], "%10d", &tmp);
        tsyncTime.doy = (unsigned int) tmp;
    
        sscanf (argv[4], "%10d", &tmp);
        tsyncTime.hours = (unsigned int) tmp;
    
        sscanf (argv[5], "%10d", &tmp);
        tsyncTime.minutes = (unsigned int) tmp;
    
        sscanf (argv[6], "%10d", &tmp);
        tsyncTime.seconds = (unsigned int) tmp;
    
        sscanf (argv[7], "%10d", &tmp);
        tsyncTime.ns = (unsigned int) tmp;

        
        printf(" Set Time DOY\n");
        err = TSYNC_CS_setTime(hnd, &tsyncTime);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set Time Seconds
    if (argc == 4)
    {
        unsigned int nSeconds;
        unsigned int nNanos;

        sscanf (argv[2], "%10d", &tmp);
        nSeconds = (unsigned int) tmp;
    
        sscanf (argv[3], "%10d", &tmp);
        nNanos = (unsigned int) tmp;

        
        printf(" Set Time DOY\n");
        err = TSYNC_CS_setTimeSec(hnd, nSeconds, nNanos);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set Time BCD
    if (argc == 9)
    {
        TSYNC_TimeBCDObj   tsyncTime = {0};
        /*
        **  read into time object
        */
        sscanf (argv[2], "%10d", &tmp);
        tsyncTime.years = (unsigned int) tmp;
    
        sscanf (argv[3], "%10d", &tmp);
        tsyncTime.doy = (unsigned int) tmp;
    
        sscanf (argv[4], "%10d", &tmp);
        tsyncTime.hours = (unsigned int) tmp;
    
        sscanf (argv[5], "%10d", &tmp);
        tsyncTime.minutes = (unsigned int) tmp;
    
        sscanf (argv[6], "%10d", &tmp);
        tsyncTime.seconds = (unsigned int) tmp;
    
        sscanf (argv[7], "%10d", &tmp);
        tsyncTime.ms = (unsigned int) tmp;

        sscanf (argv[7], "%10d", &tmp);
        tsyncTime.us = (unsigned int) tmp;

        
        printf(" Set Time BCD\n");
        err = TSYNC_CS_setTimeBcd(hnd, &tsyncTime);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    
    /*
     * close the handle to the board
     */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
