#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[32];
    char            *pgmname = argv[0];
    TSYNC_ERROR     err = TSYNC_SUCCESS;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }
    
    //Reference
    {
        printf(" GetReference\n");
        TSYNC_ReferenceObj pObj;
        err = TSYNC_SS_getRef(hnd, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            char cNone[] = "----";
            
            printf( "  Success!\n"
                    "   Time: %s\n"
                    "   PPS: %s\n",
                pObj.time[0] != '\0' ? pObj.time : cNone,
                pObj.pps[0] != '\0' ? pObj.pps : cNone);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set Max TFOM
    {
        printf(" Set Max TFOM\n");
        TFOM tfom = TFOM_3;
        err = TSYNC_SS_setMaxTfom(hnd, tfom);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get Max TFOM
    {
        printf(" Get Max TFOM\n");
        TFOM tfom;
        err = TSYNC_SS_getMaxTfom(hnd, &tfom);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Max TFOM: %u\n", tfom);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get TFOM
    {
        printf(" Get Current TFOM\n");
        TFOM tfom;
        err = TSYNC_SS_getTfom(hnd, &tfom);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Current TFOM: %u\n", tfom);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get Sync
    {
        printf(" Get Current Sync State\n");
        int bSync = 0;
        err = TSYNC_SS_getSync(hnd, &bSync);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Current Sync State: %u\n", bSync);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get Holdover
    {
        printf(" Get Current Holdover State\n");
        int bHoldover = 0;
        err = TSYNC_SS_getHoldover(hnd, &bHoldover);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Current Holdover State: %u\n", bHoldover);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Set Holdover Timeout
    {
        printf(" Set Holdover Timeout\n");
        unsigned int nHoldoverTimeout = 7300;
        err = TSYNC_SS_setHoldoverTO(hnd, nHoldoverTimeout);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Success!\n");
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get Holdover Timeout
    {
        printf(" Get Current Holdover Timeout\n");
        unsigned int nHoldoverTimeout = 0;
        err = TSYNC_SS_getHoldoverTO(hnd, &nHoldoverTimeout);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Current Holdover Timeout: %u\n", nHoldoverTimeout);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get Holdover Timestamp DOY
    {
        SS_TS_SRC src = SS_TS_TIME_REF;
        TSYNC_TimeObj pObj;
        
        printf(" Get Current Holdover Timestamp DOY (%u)\n", src);
        err = TSYNC_SS_getTimestamp(hnd, src, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "  Years: %u\n"
                    "  Day of Year: %u\n"
                    "  Hours: %u\n"
                    "  Minutes: %u\n"
                    "  Seconds: %u\n"
                    "  Nanoseconds: %u\n",
                     pObj.years,
                     pObj.doy,
                     pObj.hours,
                     pObj.minutes,
                     pObj.seconds,
                     pObj.ns);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get Holdover Timestamp BCD
    {
        SS_TS_SRC src = SS_TS_TIME_REF;
        TSYNC_TimeBCDObj pObj;
        
        printf(" Get Current Holdover Timestamp BCD (%u)\n", src);
        err = TSYNC_SS_getTimestampBcd(hnd, src, &pObj);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "  Years: %u (0x%08X)\n"
                    "  Day of Year: %u (0x%08X)\n"
                    "  Hours: %u (0x%08X)\n"
                    "  Minutes: %u (0x%08X)\n"
                    "  Seconds: %u (0x%08X)\n"
                    "  Ms: %u (0x%08X)\n"
                    "  Us: %u (0x%08X)\n",
                     pObj.years, pObj.years,
                     pObj.doy,pObj.doy,
                     pObj.hours,pObj.hours,
                     pObj.minutes,pObj.minutes,
                     pObj.seconds,pObj.seconds,
                     pObj.ms,pObj.ms,
                     pObj.us,pObj.us);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get Holdover Timestamp Seconds
    {
        SS_TS_SRC src = SS_TS_TIME_REF;
        unsigned int nSeconds;
        unsigned int nNanos;
        
        printf(" Get Current Holdover Timestamp Seconds (%u)\n", src);
        err = TSYNC_SS_getTimestampSec(hnd, src, &nSeconds, &nNanos);
        
        if (err == TSYNC_SUCCESS)
        {
            printf( "  Seconds: %u\n"
                    "  Nanos: %u\n",
                    nSeconds,
                    nNanos);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get Uptime
    {
        printf(" Get Current Uptime\n");
        unsigned int nUptime = 0;
        err = TSYNC_SS_getUptime(hnd, &nUptime);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Current Uptime: %u\n", nUptime);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
    //Get Freerun
    {
        printf(" Get Current Freerun State\n");
        int bFreerun = 0;
        err = TSYNC_SS_getFreeRun(hnd, &bFreerun);
        
        if (err == TSYNC_SUCCESS)
        {
            printf("  Current Freerun State: %u\n", bFreerun);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
        }
    }
    
        
    /*
     * close the handle to the board
     */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n", DEFAULT_DEV);
    (void)printf("\t-t   image type (optional, 0 - %u)\n", UL_IMG_NUM_ITEMS - 1);
    (void)printf("\n");
}
