
#ifndef _defined_TSYNC_AR_SERVICES_H
#define _defined_TSYNC_AR_SERVICES_H 1

/***************************************************************************
**  Module:     tsync_ar_services.h
**
**  Date:       09/08/09
**
**  Purpose:    Definitions and intermediate data structures
**              used by the driver and TSYNC API library routines
**
** (C) Copyright 2009 Spectracom Corporation  All rights reserved.
**
****************************************************************************
** Modifications:
**
**              09/08/2009 Creation
**
****************************************************************************/

#include "tsync.h"
#include "tsync_cs_services.h"

/******************************************************
**     Defines
******************************************************/

#define TSYNC_ID_AR                     0x2C
#define TSYNC_ID_AR_CA_OFFSET           0x00
#define TSYNC_ID_AR_CA_VALIDITY         0x02
#define TSYNC_ID_AR_CA_UART_CFG         0x04
#define TSYNC_ID_AR_CA_LEAP_FLAG        0x05
#define TSYNC_ID_AR_CA_LOCAL            0x06
#define TSYNC_ID_AR_CA_TIME_SCALE       0x07
#define TSYNC_ID_AR_CA_REF_ID           0x08
#define TSYNC_ID_AR_CA_FORMAT           0x09
#define TSYNC_ID_AR_CA_NUM_INST         0x0A
#define TSYNC_ID_AR_CA_PPS_SRC          0x0B  

/******************************************************
**     Define Enumerations
******************************************************/

/******************************************************
**     Define Structures
******************************************************/

#define AR_VALUE_FIELDS                         \
    TSYNC_X(        uint32_t,           value)

#define AR_OFFSET_SET_CMD_FIELDS                \
    TSYNC_X(        uint32_t,           inst)   \
    TSYNC_X(        int32_t,            offset)

#define AR_UART_CFG_OBJ_FIELDS                  \
    TSYNC_X(        UD_BR,              baud)   \
    TSYNC_X(        UD_DATA,            db)     \
    TSYNC_X(        UD_STOP,            sb)     \
    TSYNC_X(        UD_PAR,             par)

#define AR_UART_CFG_SET_CMD_FIELDS              \
    TSYNC_X(        uint32_t,           inst)   \
    TSYNC_X(        UD_BR,              baud)   \
    TSYNC_X(        UD_DATA,            db)     \
    TSYNC_X(        UD_STOP,            sb)     \
    TSYNC_X(        UD_PAR,             par)

#define AR_LOCAL_SET_CMD_FIELDS                 \
    TSYNC_X(        uint32_t,           inst)   \
    TSYNC_X(        ML_DST_REF,         ref)    \
    TSYNC_X_STRUCT( ML_DST_POINT,       in)     \
    TSYNC_X_STRUCT( ML_DST_POINT,       out)    \
    TSYNC_X(        uint32_t,           offset) \
    TSYNC_X(        int32_t,            tz)

#define AR_TIME_SCALE_SET_CMD_FIELDS            \
    TSYNC_X(        uint32_t,           inst)   \
    TSYNC_X_STRUCT( ML_TIME_SCALE_OBJ,  scale)

#define AR_FORMAT_SET_CMD_FIELDS                \
    TSYNC_X(        uint32_t,           inst)   \
    TSYNC_X(        AL_FMT,             format)

#define AR_PPS_SRC_SET_CMD_FIELDS              \
    TSYNC_X(        uint32_t,           inst)   \
    TSYNC_X(        AL_PPS,             pps)

#include "tsync_struct_define.h"

GEN_STRUCT(AR_VALUE)
GEN_STRUCT(AR_OFFSET_SET_CMD)
GEN_STRUCT(AR_UART_CFG_OBJ)
GEN_STRUCT(AR_UART_CFG_SET_CMD)
GEN_STRUCT(AR_LOCAL_SET_CMD)
GEN_STRUCT(AR_TIME_SCALE_SET_CMD)
GEN_STRUCT(AR_FORMAT_SET_CMD)
GEN_STRUCT(AR_PPS_SRC_SET_CMD)

#include "tsync_struct_undef.h"

#endif  /* _defined_TSYNC_AR_SERVICES_H */
