#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_trans.h"
#include "tsync_base_transaction.h"
#include "tsync_macros.h"

#include "tsync_hr_services.h"
#include "tsync_cs_services.h"
#include "tsync_misc_services.h"

extern uint8_t HR_VALUE_RECIPE[];
extern uint8_t EL_VALIDITY_RECIPE[];
extern uint8_t HR_VALIDITY_SET_CMD_RECIPE[];
extern uint8_t HR_DOYTIME_SET_CMD_RECIPE[];
extern uint8_t ML_CLK_LOCAL_RECIPE[];
extern uint8_t HR_LOCAL_SET_CMD_RECIPE[];
extern uint8_t ML_TIME_SCALE_OBJ_RECIPE[];
extern uint8_t HR_TIME_SCALE_SET_CMD_RECIPE[];
extern uint8_t EL_REF_ID_RECIPE[];
extern uint8_t HR_OFFSET_SET_CMD_RECIPE[];
extern uint8_t HR_MODE_SET_CMD_RECIPE[];

TSYNC_ERROR
TSYNC_HR_getValidity(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    int *bTimeValid,
    int *bPpsValid)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;

    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(bTimeValid);
    CHECK_NOT_NULL(bPpsValid);

    struct HR_VALUE inPayload;
    inPayload.value = nInstance;
    
    uint16_t ctl = 0x00;//get
    uint8_t  pos = 0;
    uint32_t pyldLen = sizeofRecipe(HR_VALUE_RECIPE, &pos);

    pos = 0;
    TSYNC_TransResult result =
        (TSYNC_TransResult)TSYNC_ALLOCA(
        SizeOfResult(sizeofRecipe(EL_VALIDITY_RECIPE, &pos)));

    err = BaseTransaction(
        TSYNC_ID_HR,
        TSYNC_ID_HR_CA_VALIDITY,
        ctl,
        pyldLen,
        (uint8_t*) (&inPayload),
        HR_VALUE_RECIPE,
        EL_VALIDITY_RECIPE,
        result,
        handle);

    CHECK_SUCCESS(err)

    struct EL_VALIDITY* outPayload =
        (EL_VALIDITY*)GetPayload(result);

    *bTimeValid = outPayload->timeValid;
    *bPpsValid = outPayload->ppsValid;

    return ( err );
}

TSYNC_ERROR
TSYNC_HR_setValidity(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    int bTimeValid,
    int bPpsValid)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;

    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

    CHECK_HANDLE(hnd);

    struct HR_VALIDITY_SET_CMD inPayload;
    inPayload.nInstance = nInstance;
    inPayload.timeValid = bTimeValid;
    inPayload.ppsValid  = bPpsValid;

    uint16_t ctl = 0x02;
    uint8_t  pos = 0;
    uint32_t pyldLen = sizeofRecipe(HR_VALIDITY_SET_CMD_RECIPE, &pos);

    TSYNC_TransResult result =
        (TSYNC_TransResult)TSYNC_ALLOCA(
        SizeOfResult(0));

    err = BaseTransaction(
        TSYNC_ID_HR,
        TSYNC_ID_HR_CA_VALIDITY,
        ctl,
        pyldLen,
        (uint8_t*) (&inPayload),
        HR_VALIDITY_SET_CMD_RECIPE,
        NULL,
        result,
        handle);

    CHECK_SUCCESS(err)

    return ( err );
}

TSYNC_ERROR
TSYNC_HR_setTime(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    TSYNC_TimeObj *pObj)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;

    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(pObj);

    struct HR_DOYTIME_SET_CMD inPayload;
    inPayload.nInstance = nInstance;
    inPayload.year = pObj->years;
    inPayload.doy = pObj->doy;
    inPayload.hour = pObj->hours;
    inPayload.minute = pObj->minutes;
    inPayload.second = pObj->seconds;
    inPayload.ns = pObj->ns;

    uint16_t ctl = 0x02;
    uint8_t  pos = 0;
    uint32_t pyldLen = sizeofRecipe(HR_DOYTIME_SET_CMD_RECIPE, &pos);

    TSYNC_TransResult result =
        (TSYNC_TransResult)TSYNC_ALLOCA(
        SizeOfResult(0));

    err = BaseTransaction(
        TSYNC_ID_HR,
        TSYNC_ID_HR_CA_TIME,
        ctl,
        pyldLen,
        (uint8_t*) (&inPayload),
        HR_DOYTIME_SET_CMD_RECIPE,
        NULL,
        result,
        handle);

    CHECK_SUCCESS(err)

    return ( err );
}

TSYNC_ERROR
TSYNC_HR_getLocal(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    TSYNC_LocalClockObj *pObj)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;

    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(pObj);

    struct HR_VALUE inPayload;
    inPayload.value = nInstance;

    uint16_t ctl = 0x00;//get
    uint8_t  pos = 0;
    uint32_t pyldLen = sizeofRecipe(HR_VALUE_RECIPE, &pos);

    pos = 0;
    TSYNC_TransResult result =
        (TSYNC_TransResult)TSYNC_ALLOCA(
        SizeOfResult(sizeofRecipe(ML_CLK_LOCAL_RECIPE, &pos)));

    err = BaseTransaction(
        TSYNC_ID_HR,
        TSYNC_ID_HR_CA_LOCAL,
        ctl,
        pyldLen,
        (uint8_t*) (&inPayload),
        HR_VALUE_RECIPE,
        ML_CLK_LOCAL_RECIPE,
        result,
        handle);

    CHECK_SUCCESS(err)

    struct ML_CLK_LOCAL* outPayload =
        (ML_CLK_LOCAL*)GetPayload(result);

    pObj->rule.ref = outPayload->ref;
    pObj->rule.in.month = outPayload->in.month;
    pObj->rule.in.wom = outPayload->in.wom;
    pObj->rule.in.dow = outPayload->in.dow;
    pObj->rule.in.hour = outPayload->in.hour;
    pObj->rule.out.month = outPayload->out.month;
    pObj->rule.out.wom = outPayload->out.wom;
    pObj->rule.out.dow = outPayload->out.dow;
    pObj->rule.out.hour = outPayload->out.hour;
    pObj->rule.offset = outPayload->offset;
    pObj->tz = outPayload->tz;

    return ( err );
}

TSYNC_ERROR
TSYNC_HR_setLocal(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    TSYNC_LocalClockObj *pObj)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;

    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(pObj);

    struct HR_LOCAL_SET_CMD inPayload;
    inPayload.inst = nInstance;
    inPayload.ref = pObj->rule.ref;
    inPayload.in.month = pObj->rule.in.month;
    inPayload.in.wom = pObj->rule.in.wom;
    inPayload.in.dow = pObj->rule.in.dow;
    inPayload.in.hour = pObj->rule.in.hour;
    inPayload.out.month = pObj->rule.out.month;
    inPayload.out.wom = pObj->rule.out.wom;
    inPayload.out.dow = pObj->rule.out.dow;
    inPayload.out.hour = pObj->rule.out.hour;
    inPayload.offset = pObj->rule.offset;
    inPayload.tz = pObj->tz;

    uint16_t ctl = 0x02;
    uint8_t  pos = 0;
    uint32_t pyldLen = sizeofRecipe(HR_LOCAL_SET_CMD_RECIPE, &pos);

    TSYNC_TransResult result =
        (TSYNC_TransResult)TSYNC_ALLOCA(
        SizeOfResult(0));

    err = BaseTransaction(
        TSYNC_ID_HR,
        TSYNC_ID_HR_CA_LOCAL,
        ctl,
        pyldLen,
        (uint8_t*) (&inPayload),
        HR_LOCAL_SET_CMD_RECIPE,
        NULL,
        result,
        handle);

    CHECK_SUCCESS(err)

    return ( err );
}

TSYNC_ERROR
TSYNC_HR_getTimeScale(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    TSYNC_TimeScaleObj *pObj)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;

    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(pObj);

    struct HR_VALUE inPayload;
    inPayload.value = nInstance;

    uint16_t ctl = 0x00;//get
    uint8_t  pos = 0;
    uint32_t pyldLen = sizeofRecipe(HR_VALUE_RECIPE, &pos);

    pos = 0;
    TSYNC_TransResult result =
        (TSYNC_TransResult)TSYNC_ALLOCA(
        SizeOfResult(sizeofRecipe(ML_TIME_SCALE_OBJ_RECIPE, &pos)));

    err = BaseTransaction(
        TSYNC_ID_HR,
        TSYNC_ID_HR_CA_TIME_SCALE,
        ctl,
        pyldLen,
        (uint8_t*) (&inPayload),
        HR_VALUE_RECIPE,
        ML_TIME_SCALE_OBJ_RECIPE,
        result,
        handle);

    CHECK_SUCCESS(err)

    struct ML_TIME_SCALE_OBJ* outPayload =
        (ML_TIME_SCALE_OBJ*)GetPayload(result);

    pObj->scale = outPayload->scale;

    return ( err );
}

TSYNC_ERROR
TSYNC_HR_setTimeScale(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    TSYNC_TimeScaleObj *pObj)
{
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(pObj);

    struct HR_TIME_SCALE_SET_CMD inPayload;
    inPayload.inst = nInstance;
    inPayload.scale.scale = pObj->scale;

    uint16_t ctl = 0x02;//set
    uint8_t  pos = 0;
    uint32_t pyldLen = sizeofRecipe(HR_TIME_SCALE_SET_CMD_RECIPE, &pos);

    TSYNC_TransResult result =
        (TSYNC_TransResult)TSYNC_ALLOCA(
        SizeOfResult(0));

    TSYNC_ERROR err = BaseTransaction(
        TSYNC_ID_HR,
        TSYNC_ID_HR_CA_TIME_SCALE,
        ctl,
        pyldLen,
        (uint8_t*) (&inPayload),
        HR_TIME_SCALE_SET_CMD_RECIPE,
        NULL,
        result,
        handle);

    return ( err );
}

TSYNC_ERROR
TSYNC_HR_getRefId(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    TSYNC_RefIdObj *pObj)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(pObj);

    struct HR_VALUE inPayload;
    inPayload.value = nInstance;
    
    uint16_t ctl = 0x00;
    uint8_t  pos = 0;
    uint32_t pyldLen = sizeofRecipe(HR_VALUE_RECIPE, &pos);

    pos = 0;
    TSYNC_TransResult result =
        (TSYNC_TransResult)TSYNC_ALLOCA(
        SizeOfResult(sizeofRecipe(EL_REF_ID_RECIPE, &pos)));
    
    err = BaseTransaction(
        TSYNC_ID_HR,
        TSYNC_ID_HR_CA_REF_ID,
        ctl,
        pyldLen,
        (uint8_t*) (&inPayload),
        HR_VALUE_RECIPE,
        EL_REF_ID_RECIPE,
        result,
        handle);

    CHECK_SUCCESS(err)
    
    struct EL_REF_ID* outPayload =
        (EL_REF_ID*)GetPayload(result);
        
    memset(pObj->refid, '\0', sizeof(pObj->refid));
    memcpy(pObj->refid, outPayload->ref, sizeof(outPayload->ref));
        
    return ( err );
}

TSYNC_ERROR
TSYNC_HR_getNumInst(
    TSYNC_BoardHandle hnd,
    unsigned int *nInstances)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;

    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(nInstances);

    uint16_t ctl = 0x00;//get
    uint8_t  pos = 0;
    uint32_t pyldLen = 0;

    TSYNC_TransResult result =
        (TSYNC_TransResult)TSYNC_ALLOCA(
        SizeOfResult(sizeofRecipe(HR_VALUE_RECIPE, &pos)));

    err = BaseTransaction(
        TSYNC_ID_HR,
        TSYNC_ID_HR_CA_NUM_INST,
        ctl,
        pyldLen,
        NULL,
        NULL,
        HR_VALUE_RECIPE,
        result,
        handle);

    CHECK_SUCCESS(err)

    struct HR_VALUE* outPayload =
        (HR_VALUE*)GetPayload(result);

    *nInstances = outPayload->value;

    return ( err );
}

TSYNC_ERROR
TSYNC_HR_getOffset(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    int *nOffset)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(nOffset);

        struct HR_VALUE inPayload;
        inPayload.value = nInstance;
        
        uint16_t ctl = 0x00;//get
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(HR_VALUE_RECIPE, &pos);

        pos = 0;
        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(sizeofRecipe(HR_VALUE_RECIPE, &pos)));
        
        err = BaseTransaction(
            TSYNC_ID_HR,
            TSYNC_ID_HR_CA_OFFSET,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            HR_VALUE_RECIPE,
            HR_VALUE_RECIPE,
            result,
            handle);

        CHECK_SUCCESS(err)
        
        struct HR_VALUE* outPayload =
            (HR_VALUE*)GetPayload(result);
            
        *nOffset = outPayload->value;
        
    return ( err );
}

TSYNC_ERROR
TSYNC_HR_setOffset(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    int nOffset)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);

        struct HR_OFFSET_SET_CMD inPayload;
        inPayload.inst = nInstance;
        inPayload.offset = nOffset;
        
        uint16_t ctl = 0x02;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(HR_OFFSET_SET_CMD_RECIPE, &pos);

        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(0));
        
        err = BaseTransaction(
            TSYNC_ID_HR,
            TSYNC_ID_HR_CA_OFFSET,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            HR_OFFSET_SET_CMD_RECIPE,
            NULL,
            result,
            handle);

        CHECK_SUCCESS(err)
        
    return ( err );
}

TSYNC_ERROR
TSYNC_HR_getMode(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    HR_TS_MODE *nMode)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);
        CHECK_NOT_NULL(nMode);

        struct HR_VALUE inPayload;
        inPayload.value = nInstance;
        
        uint16_t ctl = 0x00;//get
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(HR_VALUE_RECIPE, &pos);

        pos = 0;
        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(sizeofRecipe(HR_VALUE_RECIPE, &pos)));
        
        err = BaseTransaction(
            TSYNC_ID_HR,
            TSYNC_ID_HR_CA_MODE,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            HR_VALUE_RECIPE,
            HR_MODE_SET_CMD_RECIPE,
            result,
            handle);

        CHECK_SUCCESS(err)
        
        struct HR_VALUE* outPayload =
            (HR_VALUE*)GetPayload(result);
            
        *nMode = outPayload->value;
        
    return ( err );
}

TSYNC_ERROR
TSYNC_HR_setMode(
    TSYNC_BoardHandle hnd,
    unsigned int nInstance,
    HR_TS_MODE nMode)
{
    TSYNC_ERROR err = TSYNC_SUCCESS;
    
    TSYNC_BoardObj *handle = (TSYNC_BoardObj*)hnd;

        CHECK_HANDLE(hnd);

        struct HR_MODE_SET_CMD inPayload;
        inPayload.inst = nInstance;
        inPayload.mode = nMode;
        
        uint16_t ctl = 0x02;
        uint8_t  pos = 0;
        uint32_t pyldLen = sizeofRecipe(HR_MODE_SET_CMD_RECIPE, &pos);

        TSYNC_TransResult result =
            (TSYNC_TransResult)TSYNC_ALLOCA(
            SizeOfResult(0));
        
        err = BaseTransaction(
            TSYNC_ID_HR,
            TSYNC_ID_HR_CA_MODE,
            ctl,
            pyldLen,
            (uint8_t*) (&inPayload),
            HR_MODE_SET_CMD_RECIPE,
            NULL,
            result,
            handle);

        CHECK_SUCCESS(err)
        
    return ( err );
}
