#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                i;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    AL_FMT             fmt [AL_FMT_MAX];


    /* If invalid number of arguments... */
    if ((argc < 4) || (argc > 6))
    {
        printf(" Usage: AP_SetFormat <device index> <index> <fmt0> [fmt1] [fmt2]\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Format message */
    index  = atoi(argv[2]);
    fmt[0] = (AL_FMT)atoi(argv[3]);
    fmt[1] = (argc >= 5) ? (AL_FMT)atoi(argv[4]) : AL_FMT_UNKNOWN;
    fmt[2] = (argc >= 6) ? (AL_FMT)atoi(argv[5]) : AL_FMT_UNKNOWN;

    for (i = 0; i < AL_FMT_MAX; i++)
    {
        printf("\n  AP (%d) Format %d: (%d) ", (index), (i), (fmt[i]));

        switch ((fmt[i]))
        {
            case AL_FMT_SPEC_0:   printf("Spectracom format 0\n");    break;
            case AL_FMT_SPEC_1:   printf("Spectracom format 1\n");    break;
            case AL_FMT_SPEC_2:   printf("Spectracom format 2\n");    break;
            case AL_FMT_SPEC_3:   printf("Spectracom format 3\n");    break;
            case AL_FMT_SPEC_4:   printf("Spectracom format 4\n");    break;
            case AL_FMT_SPEC_5:            // Unsupported
            case AL_FMT_SPEC_6:   ret = 1; // Unsupported
                                  printf("Unsupported\n");            break;
            case AL_FMT_SPEC_7:   printf("Spectracom format 7\n");    break;
            case AL_FMT_SPEC_8:   printf("Spectracom format 8\n");    break;
            case AL_FMT_SPEC_9:   printf("Spectracom format 9\n");    break;
            case AL_FMT_SPEC_1S:  printf("Spectracom format 1S\n");   break;
            case AL_FMT_SPEC_9S:  printf("Spectracom format 9S\n");   break;
            case AL_FMT_NMEA_GGA: printf("NMEA GGA message\n");       break;
            case AL_FMT_NMEA_RMC: printf("NMEA RMC message\n");       break;
            case AL_FMT_NMEA_ZDA: printf("NMEA ZDA message\n");       break;
            case AL_FMT_BBC_01:   printf("BBC-01 format\n");          break;
            case AL_FMT_BBC_02:   printf("BBC-02 format\n");          break;
            case AL_FMT_BBC_03:   printf("BBC-03 format\n");          break;
            case AL_FMT_BBC_04:   printf("BBC-04 format\n");          break;
            case AL_FMT_BBC_05:   printf("BBC-05 format (RMC)\n");    break;
            case AL_FMT_153C_BB:  printf("Buffer Box (253)\n");       break;
            case AL_FMT_153C_TT:  printf("Time Transfer (5101)\n");   break;
            case AL_FMT_153C_CS:  printf("Current Status (5040)\n");  break;
            case AL_FMT_153C_TM:  printf("Time Mark Data (3)\n");     break;
            case AL_FMT_ER_0:     printf("ER format 0\n");            break;
            case AL_FMT_ER_1:     printf("ERX format 1\n");           break;
            case AL_FMT_EVT_0:    printf("Event Timestamp Format 0"); break;
            case AL_FMT_EVT_1:    printf("Event Timestamp Format 1"); break;
            case AL_FMT_EPS_1:    printf("EPSILON TOD1 format\n");    break;
            case AL_FMT_EPS_3:    printf("EPSILON TOD3 format\n");    break;
            case AL_FMT_UNKNOWN:  printf("None\n");                   break;
            default:              printf("Unknown\n");                break;
        }
    }

    // if an error is detected do NOT send the command
    if (ret == 0)
    {
        // Send Set Format transaction
        err = TSYNC_AP_setFormat(hnd, (IO_PORT)index, fmt);

        if (err != TSYNC_SUCCESS)
        {
            printf("  Error: %s.\n", tsync_strerror(err));
            return (1);
        }
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
