#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    unsigned int       num = 0;
    char               req = 'T';


    /* 
     *  If invalid number of arguments or MORE than 1 Request Character or the 
     *  request character is in 0x## hex format
    */
    if ((argc != 4) || 
        ((strlen(argv[3]) != 1) && (strlen(argv[3]) != 4)))
    {
        printf(" Usage: AP_SetReqChar <device index> <index> <req char or 0x## format>\n");
        return (1);
    }

    /*
     * Verify hex format 
     */
    if ((strlen(argv[3]) == 4) && 
        ((argv[3][0] != '0') || ((argv[3][1] != 'x') && (argv[3][1] != 'X'))))
    {
        printf(" Error: Not in valid hex format 0x##>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Request Character message */
    index = atoi(argv[2]);
 
    if (strlen(argv[3]) == 1)
    {
        req = (char)argv[3][0];
        num = (unsigned int)req;
    }
    else
    {
        sscanf(argv[3], "0%*c%2x", &num);
        req = (char)num;

        if (num > 255)
        {
            printf("  Error: num=%d - Too large > 255\n", num);
            return (1);
        }
    }

    printf("\n  AP (%d) Request Character: (%c) 0x%02x\n", (index), (req), num);

    // Send Set Format transaction
    err = TSYNC_AP_setReqChar(hnd, index, req);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
