#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    AL_FMT             fmt;


    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: AR_GetFormat <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Format message */
    index = atoi(argv[2]);

    // Send Get Format transaction
    err = TSYNC_AR_getFormat(hnd, index, &fmt);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) &&
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%d\n", fmt);
    }

    printf("\n  AR (%d) Format: (%d) ", (index), (fmt));

    switch ((fmt))
    {
        case AL_FMT_SPEC_0:    printf("Spectracom format 0\n");    break;
        case AL_FMT_SPEC_1:    printf("Spectracom format 1\n");    break;
        case AL_FMT_SPEC_2:    printf("Spectracom format 2\n");    break;
        case AL_FMT_SPEC_3:    printf("Spectracom format 3\n");    break;
        case AL_FMT_SPEC_4:    printf("Spectracom format 4\n");    break;
        case AL_FMT_SPEC_7:    printf("Spectracom format 7\n");    break;
        case AL_FMT_SPEC_8:    printf("Spectracom format 8\n");    break;
        case AL_FMT_SPEC_9:    printf("Spectracom format 9\n");    break;
        case AL_FMT_SPEC_1S:   printf("Spectracom format 1S\n");   break;
        case AL_FMT_SPEC_9S:   printf("Spectracom format 9S\n");   break;
        case AL_FMT_NMEA_GGA:  printf("NMEA GGA message\n");       break;
        case AL_FMT_NMEA_RMC:  printf("NMEA RMC message\n");       break;
        case AL_FMT_NMEA_ZDA:  printf("NMEA ZDA message\n");       break;
        case AL_FMT_BBC_01:    printf("BBC-01 format\n");          break;
        case AL_FMT_BBC_02:    printf("BBC-02 format\n");          break;
        case AL_FMT_BBC_03:    printf("BBC-03 format\n");          break;
        case AL_FMT_BBC_04:    printf("BBC-04 format\n");          break;
        case AL_FMT_BBC_05:    printf("BBC-05 (RMC) format\n");    break;
        case AL_FMT_153C_BB:   printf("ICD-153C 253 Buffer Box\n");
                                                                   break;
        case AL_FMT_153C_TT:   printf("ICD-153C 5101 Time Transfer\n");
                                                                   break;
        case AL_FMT_153C_CS:   printf("ICD-153C 5040 Current Status\n");
                                                                   break;
        case AL_FMT_153C_TM:   printf("ICD-153C 3 Time Mark Data\n");
                                                                   break;
        case AL_FMT_ER_0:      printf("ER format 0\n");            break;
        case AL_FMT_ER_1:      printf("ERX format 1\n");           break;

        case AL_FMT_EL_UTC:    printf("E-Loran UTC format\n");     break;
        case AL_FMT_EL_LCRSDA: printf("E-Loran LCRSDA format\n");  break;

        case AL_FMT_UNKNOWN:   printf("Auto-Detect\n");            break;
        default:               printf("Unknown\n");                break;
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */

