#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    AL_PPS             pps;


    /* If invalid number of arguments... */
    if (argc != 4)
    {
        printf(" Usage: AR_Set1PpsSrc <device index> <index> <OTP=0|Sig=1>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Format message */
    index = atoi(argv[2]);
    pps   = (AL_FMT)atoi(argv[3]);

    printf("\n  AR (%d) 1PPS Source: (%d) ", (index), (pps));

    switch (pps)
    {
        case AL_PPS_OTP:   printf("Message On-Time Point\n"); break;
        case AL_PPS_SIG:   printf("Input 1PPS Signal\n");     break;
        default:           printf("Unknown\n");               break;
    }

    // if an error is detected do NOT send the command
    if (ret == 0)
    {
        // Send Set 1PPS Source transaction
        err = TSYNC_AR_setPpsSrc(hnd, index, pps);

        if (err != TSYNC_SUCCESS)
        {
            printf("  Error: %s.\n", tsync_strerror(err));
            return (1);
        }
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
