#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle    hnd;
    int                  rc;
    int                  ret     = 0;
    int                  devIdx;
    char                *devName = DEFAULT_DEV;
    char                 fullDevName[32];
    char                *pgmname = argv[0];
    TSYNC_ERROR          err     = TSYNC_SUCCESS;
    TSYNC_TimeDiscontObj disco;

    /* If invalid number of arguments... */
    if (argc != 2)
    {
        printf(" Usage: CS_GetTimeDiscont <device index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Get Time Discontinuity */
    err = TSYNC_CS_getTimeDiscont(hnd, &disco);
        
    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

     /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) && 
          *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%u %u %u %u %u %u %u %u %u %u %u\n", 
            (disco.bActive),
            (disco.effectiveTime.hours), (disco.effectiveTime.minutes),
            (disco.effectiveTime.seconds), (disco.effectiveTime.doy),
            (disco.effectiveTime.years),
            (disco.newTime.hours), (disco.newTime.minutes), 
            (disco.newTime.seconds), (disco.newTime.doy), 
            (disco.newTime.years));
    }

    printf("\n");
    printf(" Time Discontinuity: %s\n",
           ((disco.bActive) == 1) ? "Active" : "Inactive");
    printf("   %02u:%02u:%02u %03u/%04u --> %02u:%02u:%02u %03u/%04u\n",
           (disco.effectiveTime.hours), (disco.effectiveTime.minutes),
           (disco.effectiveTime.seconds), (disco.effectiveTime.doy),
           (disco.effectiveTime.years),
           (disco.newTime.hours), (disco.newTime.minutes), 
           (disco.newTime.seconds), (disco.newTime.doy), 
           (disco.newTime.years));

    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
