#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle      hnd;
    int                    rc;
    int                    ret     = 0;
    int                    devIdx;
    char                  *devName = DEFAULT_DEV;
    char                   fullDevName[32];
    char                  *pgmname = argv[0];
    TSYNC_ERROR            err     = TSYNC_SUCCESS;
    TSYNC_TimeDSTStateObj  ds;


    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: CS_SetDstState <device index> <DST state (1=IN, 0=OUT)>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get DST State message */
    ds.state = atoi(argv[2]);

    printf("\n");
    printf(" DST State: %s Time\n", (ds.state == 1) ? "Daylight" : "Standard");

    /* Set DST State */
    err = TSYNC_CS_setDstState(hnd, &ds);
        
    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        ret = (strstr(tsync_strerror(err), "RC_CHG_PENDING") != NULL) ? 2 : 1;
    }

    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
