#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int                 rc;
    int                 ret     = 0;
    int                 devIdx;
    char               *devName = DEFAULT_DEV;
    char                fullDevName[32];
    char               *pgmname = argv[0];
    TSYNC_ERROR         err     = TSYNC_SUCCESS;
    unsigned int        slot;
    TSYNC_OptCardHdrObj info;


    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: DCS_GetCardInfo <device index> <slot>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Local message */
    slot = atoi(argv[2]);

    // Send Get Local transaction
    err = TSYNC_DCS_getCardInfo(hnd, slot, &info);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }
    
    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) && 
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%d %d %d %d %d\n", info.id, info.ver, info.cish.pldId,
               info.cish.pldId, info.cish.num);
    }

    printf("\n  DCS option card info for slot %u:\n", (slot));

    printf("\n    Card ID:                 0x%02X\n", info.id);

    printf("\n    Card Version:            0x%02X\n", info.ver);

    printf("\n    FPGA ID:                 0x%02X\n", info.cish.pldId);

    printf("\n    FPGA Version:            0x%04X\n", info.cish.pldVer);

    printf("\n    # of Feature Records:    %d\n", info.cish.num);

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
