#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    TSYNC_TimeScaleObj ts;
    char               tsStr[8];


    /* If invalid number of arguments... */
    if (argc != 4)
    {
        printf(" Usage: DP_SetTimeScale <device index> <index> <timescale>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Format message */
    index    = atoi(argv[2]);
    ts.scale = (ML_TIME_SCALE)atoi(argv[3]);

    switch ((ts.scale))
    {
        case ML_TIME_SCALE_UTC:   strcpy(tsStr, "UTC");     break;
        case ML_TIME_SCALE_TAI:   strcpy(tsStr, "TAI");     break;
        case ML_TIME_SCALE_GPS:   strcpy(tsStr, "GPS");     break;
        case ML_TIME_SCALE_LOCAL: strcpy(tsStr, "LOCAL");   break;
        default:                  strcpy(tsStr, "Unknown"); break;
    }

    printf("\n  DP (%d) Timescale: %s (%d)\n", (index), (tsStr), (ts.scale));

    // Send Set Format transaction
    err = TSYNC_DP_setTimeScale(hnd, (IO_PORT)index, &ts);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
