#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    LE_INDEX           led;
    EC_STATE           state;


    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: EC_GetState <device index> <led>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get state message */
    led = (LE_INDEX)atoi(argv[2]);

    /* Get LED State */
    err = TSYNC_EC_getState(hnd, led, &state);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }
    
     /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) && 
          *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%d\n", state);
    }

    printf("\n");
    printf(" LED %d state: ", (led));

    switch ((state))
    {
        case EC_STATE_OFF:   printf("(%u) Off\n",     (state)); break;
        case EC_STATE_ON:    printf("(%u) On\n",      (state)); break;
        case EC_STATE_BLINK: printf("(%u) Blink\n",   (state)); break;
        case EC_STATE_CODE:  printf("(%u) Code\n",    (state)); break;
        default:             printf("(%u) Unknown\n", (state)); break;
    }

    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
