#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    LE_INDEX           led;
    EC_MODE            mode;


    /* If invalid number of arguments... */
    if (argc != 4)
    {
        printf(" Usage: EC_SetMode <device index> <led> <mode>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Mode message */
    led  = (LE_INDEX)atoi(argv[2]);
    mode = (EC_MODE)atoi(argv[3]);

    printf("\n");
    printf(" LED %d mode: ", (led));

    switch ((mode))
    {
        case EC_MODE_SYNC:     printf("(%u) Sync\n",     (mode)); break;
        case EC_MODE_HOLDOVER: printf("(%u) Holdover\n", (mode)); break;
        case EC_MODE_ALARM:    printf("(%u) Alarm\n",    (mode)); break;
        case EC_MODE_1PPS:     printf("(%u) 1PPS\n",     (mode)); break;
        case EC_MODE_MANUAL:   printf("(%u) Manual\n",   (mode)); break;
        default:               printf("(%u) Unknown\n",  (mode)); break;
    }

    /* Set LED Mode */
    err = TSYNC_EC_setMode(hnd, led, mode);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
