#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    ETO_CFG            cfg;


    memset(&cfg, 0, sizeof(ETO_CFG));

    /* If invalid number of arguments... */
    if ((argc < 4) || (argc > 8))
    {
        printf(" Usage: ETP_SetCfg <device index> <index> <mode> [framing(E1,T1)] [encoding(T1)] [ssm enable(E1,T1)] [ssm value(E1,T1)]\n");
        return (1);
    }
    else
    {
        // Convert the mode
        cfg.mode = (ETO_MODE)atoi(argv[3]);

        switch (cfg.mode)
        {
            case ETO_MODE_T1:
                if (argc != 8)
                {
                    printf(" Usage: ETP_SetCfg <device index> <index> 0 <framing> <encoding> <ssm enable> <ssm value>\n");
                    return (1);
                }
                break;
            case ETO_MODE_E1:
                if (argc != 7)
                {
                    printf(" Usage: ETP_SetCfg <device index> <index> 1 <framing> <ssm enable> <ssm value>\n");
                    return (1);
                }
                break;
            case ETO_MODE_DISABLE:
                if (argc != 4)
                {
                    printf(" Usage: ETP_SetCfg <device index> <index> 2\n");
                    return (1);
                }
                break;
            default:
                printf(" Invalid mode %d (0=T1, 1=E1, 2=Disable)\n", cfg.mode);
                return (1);
        }
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Format message */
    index = atoi(argv[2]);

    if (cfg.mode == ETO_MODE_T1)
    {
        cfg.t1Frame  = (ETO_T1_FRM)atoi(argv[4]);
        cfg.t1Encode = (ETO_T1_ENC)atoi(argv[5]);
        cfg.ssmEn    = (int)atoi(argv[6]);
        cfg.t1Ssm    = (ETO_T1_SSM)atoi(argv[7]);
    }
    else if (cfg.mode == ETO_MODE_E1)
    {
        cfg.e1Frame  = (ETO_E1_FRM)atoi(argv[4]);
        cfg.ssmEn    = (int)atoi(argv[5]);
        cfg.e1Ssm    = (ETO_E1_SSM)atoi(argv[6]);
    }

    printf("\n  ETP (%d) Configuration:\n", (index));
    printf("\n      Operational Mode:    (%d) ", (cfg.mode));

    switch ((cfg.mode))
    {
        case ETO_MODE_T1:        printf("T1\n");           break;
        case ETO_MODE_E1:        printf("E1\n");           break;
        case ETO_MODE_DISABLE:   printf("Disabled\n");     break;
        default:                 printf("Unknown\n");      break;
    }

    if (cfg.mode == ETO_MODE_E1)
    {
        printf("\n      E1 Framing Type:     (%d) ", (cfg.e1Frame));

        switch ((cfg.e1Frame))
        {
            case ETO_E1_FRM_CRC4:        printf("CRC4\n");          break;
            case ETO_E1_FRM_NO_CRC4:     printf("No CRC4\n");       break;
            case ETO_E1_FRM_AIS:         printf("AIS\n");           break;
            default:                     printf("Unknown\n");       break;
        }

        if (cfg.ssmEn == 1)
        {                                        
            printf("\n      E1 SSM Level:        (%d) ", (cfg.e1Ssm));

            switch ((cfg.e1Ssm))
            {
                case ETO_E1_SSM_UNK:    printf("QL-UNK\n");        break;
                case ETO_E1_SSM_PRC:    printf("QL-PRC\n");        break;
                case ETO_E1_SSM_SSU_A:  printf("QL-SSU-A\n");      break;
                case ETO_E1_SSM_SSU_B:  printf("QL-SSU-B\n");      break;
                case ETO_E1_SSM_SEC:    printf("QL-SEC\n");        break;
                case ETO_E1_SSM_DNU:    printf("QL-DNU\n");        break;
                default:                printf("Unknown\n");       break;
            }
        }
        else
        {
            printf("\n      E1 SSM Level:        Disabled\n");
        }
    }
    else if (cfg.mode == ETO_MODE_T1)
    {
        printf("\n      T1 Framing Type:     (%d) ", (cfg.t1Frame));

        switch ((cfg.t1Frame))
        {
            case ETO_T1_FRM_D4SF:        printf("SF\n");            break;
            case ETO_T1_FRM_ESF_CRC6:    printf("ESF CRC6\n");      break;
            case ETO_T1_FRM_ESF_NO_CRC6: printf("ESF no CRC6\n");   break;
            case ETO_T1_FRM_AIS:         printf("AIS\n");           break;
            default:                     printf("Unknown\n");       break;
        }
        
        printf("\n      T1 Encoding Type:    (%d) ", (cfg.t1Encode));

        switch ((cfg.t1Encode))
        {
            case ETO_T1_ENC_B8ZS:        printf("B8ZS\n");          break;
            case ETO_T1_ENC_AMI:         printf("AMI\n");           break;
            default:                     printf("Unknown\n");       break;
        }

        if (cfg.ssmEn == 1)
        {                                        
            printf("\n      T1 SSM Level:        (%d) ", (cfg.e1Ssm));

            switch ((cfg.t1Ssm))
            {
                case ETO_T1_SSM_PRS:    printf("QL-PRS\n");        break;
                case ETO_T1_SSM_STU:    printf("QL-STU\n");        break;
                case ETO_T1_SSM_ST2:    printf("QL-ST2\n");        break;
                case ETO_T1_SSM_TNC:    printf("QL-TNC\n");        break;
                case ETO_T1_SSM_ST3E:   printf("QL-ST3E\n");       break;
                case ETO_T1_SSM_ST3:    printf("QL-ST3\n");        break;
                case ETO_T1_SSM_SMC:    printf("QL-SMC\n");        break;
                case ETO_T1_SSM_ST4:    printf("QL-ST4\n");        break;
                case ETO_T1_SSM_RES:    printf("QL-RES\n");        break;
                case ETO_T1_SSM_DUS:    printf("QL-DUS\n");        break;
                default:                printf("Unknown\n");       break;
            }
        }
        else
        {
            printf("\n      T1 SSM Level:        Disabled\n");
        }
    }

    // Send Set Format transaction
    err = TSYNC_ETP_setCfg(hnd, index, &cfg);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
