#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    OD_PIN             pin;
    LEVEL              lvl;
    int                en;


    /* If invalid number of arguments... */
    if (argc != 5)
    {
        printf(" Usage: GO_SetMatchEnable <device index> <pin> <lvl>");
        printf(" <enable>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set match enable message */
    pin = (OD_PIN)atoi(argv[2]);
    lvl = (LEVEL)atoi(argv[3]);
    en  = atoi(argv[4]);

    printf("\n");
    printf(" GO (%d) Match %s Enable: %d\n", (pin),
           ((lvl == LEVEL_HIGH) ? "High" : "Low"), (en));

    // Send Set match enable transaction
    err = TSYNC_GO_setMatchEnable(hnd, pin, lvl, en);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
