#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int                 rc;
    int                 ret     = 0;
    int                 devIdx;
    char               *devName = DEFAULT_DEV;
    char                fullDevName[32];
    char               *pgmname = argv[0];
    TSYNC_ERROR         err     = TSYNC_SUCCESS;
    OD_PIN              pin;
    TSYNC_GPOSquareObj  swCfg;
    unsigned int        localPer;
    unsigned int        localScale;


    /* If invalid number of arguments... */
    if (argc != 8)
    {
        printf(" Usage: GO_SetSqWave <device index> <pin> <offset> <period>");
        printf(" <period scale> <pulse width> <active edge>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set square wave message */
    pin        = (OD_PIN)atoi(argv[2]);
    swCfg.off  = atoi(argv[3]);
    localPer   = atoi(argv[4]);
    localScale = atoi(argv[5]);
    swCfg.pw   = atoi(argv[6]);
    swCfg.ae   = (EDGE)atoi(argv[7]);

    if (localScale == 0)
    {
        swCfg.per = localPer;
    }
    else
    {
        swCfg.per =  0x80000000;
        swCfg.per += localPer;
    }

    printf("\n");
    printf(" GO (%d) Square Wave Configuration:\n", (pin));
    printf("  Offset:      %d ns\n", (swCfg.off));
    printf("  Period:      %u ", (swCfg.per & 0x7FFFFFFF));
    if (localScale == 0)
    {
        printf("ns\n");
    }
    else
    {
        printf("us\n");
    }
    printf("  Pulse Width: %u ns\n", (swCfg.pw));
    printf("  Active Edge: %u\n", (swCfg.ae));

    // Send Set square wave transaction
    err = TSYNC_GO_setSquareWave(hnd, pin, &swCfg);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
