#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    TSYNC_GplIpV4      currentIPV4;

    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: GPR_GetCurrentIPV4 <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    // Build Get Clock Identity Message
    index = atoi(argv[2]);

    // Send Get Number of Instances Transaction
    err = TSYNC_GPR_getCurrentIPV4(hnd, index, &currentIPV4);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) &&
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%i.%i.%i.%i %i.%i.%i.%i %i.%i.%i.%i\n",
                currentIPV4.ipV4Addr[0],
                currentIPV4.ipV4Addr[1],
                currentIPV4.ipV4Addr[2],
                currentIPV4.ipV4Addr[3],
                currentIPV4.gateway[0],
                currentIPV4.gateway[1],
                currentIPV4.gateway[2],
                currentIPV4.gateway[3],
                currentIPV4.netmask[0],
                currentIPV4.netmask[1],
                currentIPV4.netmask[2],
                currentIPV4.netmask[3]);
    }

    printf("\n");
    printf(" GPR%i Static IP Address: %i.%i.%i.%i \n",
                (index),
                currentIPV4.ipV4Addr[0],
                currentIPV4.ipV4Addr[1],
                currentIPV4.ipV4Addr[2],
                currentIPV4.ipV4Addr[3]);
    printf(" Gateway               : %i.%i.%i.%i \n",
                currentIPV4.gateway[0],
                currentIPV4.gateway[1],
                currentIPV4.gateway[2],
                currentIPV4.gateway[3]);
    printf(" Netmask               : %i.%i.%i.%i \n",
                currentIPV4.netmask[0],
                currentIPV4.netmask[1],
                currentIPV4.netmask[2],
                currentIPV4.netmask[3]);


    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
