#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    TSYNC_GplMsgRates  msgRates;

    /* If invalid number of arguments... */
    if (argc != 6)
    {
        printf(" Usage: GPR_SetMsgRates <device index> <index> <Ann Int> <Sync Int> <Delay Req Int>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    // Build Set Clock Identity Message
    index = atoi(argv[2]);
    msgRates.logAnnInterval      = atoi(argv[3]);
    msgRates.logSyncInterval     = atoi(argv[4]);
    msgRates.logDelayReqInterval = atoi(argv[5]);

    // Send Set Transaction
    err = TSYNC_GPR_setMsgRates(hnd, index, &msgRates);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) &&
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%i %i %i \n", msgRates.logAnnInterval,
                msgRates.logSyncInterval,
                msgRates.logDelayReqInterval);
    }

    printf("\n");
    printf(" GPR%i Message Rates     \n", index);
    printf(" Announce Interval      : %i \n", msgRates.logAnnInterval);
    printf(" Sync Interval          : %i \n", msgRates.logSyncInterval);
    printf(" Delay Request Interval : %i \n", msgRates.logDelayReqInterval);


    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
