#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle       hnd;
    int                     rc;
    int                     ret     = 0;
    int                     devIdx;
    char                   *devName = DEFAULT_DEV;
    char                    fullDevName[32];
    char                   *pgmname = argv[0];
    TSYNC_ERROR             err     = TSYNC_SUCCESS;
    unsigned int            index;
    TSYNC_GplUnctMasterCfg  masterCfg;

    /* If invalid number of arguments... */
    if (argc != 10)
    {
        printf(" Usage: GPR_SetUnctMasterCfg <device index> <index> <min ann int> <max ann dur> <min sync int> <max sync dur> <min DRq int> <max DRq dur> <Max Slaves>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    // Build Set Clock Identity Message
    index = atoi(argv[2]);
    masterCfg.minAnnInt  = atoi(argv[3]);
    masterCfg.maxAnnDur  = atoi(argv[4]);
    masterCfg.minSyncInt = atoi(argv[5]);
    masterCfg.maxSyncDur = atoi(argv[6]);
    masterCfg.minDRqInt  = atoi(argv[7]);
    masterCfg.maxDRqDur  = atoi(argv[8]);
    masterCfg.maxSlaves  = atoi(argv[9]);


    // Send Set Transaction
    err = TSYNC_GPR_setUnctMasterCfg(hnd, index, &masterCfg);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) &&
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%hhi %i %hhi %i %hhi %i %i \n",
                masterCfg.minAnnInt,
                masterCfg.maxAnnDur,
                masterCfg.minSyncInt,
                masterCfg.maxSyncDur,
                masterCfg.minDRqInt,
                masterCfg.maxDRqDur,
                masterCfg.maxSlaves);
    }

    printf("\n");
    printf(" GPR%i Master Config         \n",(index));
    printf(" Min Announce Interval      : %hhi \n", masterCfg.minAnnInt);
    printf(" Max Announce Duration      : %i \n", masterCfg.maxAnnDur);
    printf(" Min Sync Interval          : %hhi \n", masterCfg.minSyncInt);
    printf(" Max Sync Duration          : %i \n", masterCfg.maxSyncDur);
    printf(" Min Delay Request Interval : %hhi \n", masterCfg.minDRqInt);
    printf(" Max Delay Request Duration : %i \n", masterCfg.maxDRqDur);
    printf(" Max Slaves                 : %i \n", masterCfg.maxSlaves);

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
