#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    TSYNC_AlmObj       almanac;

    /* If invalid number of arguments... */
    if (argc != 4)
    {
        printf(" Usage: GR_GetAlm <device index> <index> <svID>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Almanac message */
    index = atoi(argv[2]);

    almanac.alm.prn = atoi(argv[3]);

    // Send Get Almanac
    err = TSYNC_GR_getAlm(hnd, index, &almanac);

    if (err != TSYNC_SUCCESS)
    {
        // Check the error code to determine if the error is due to not having
        // data for the specified SV.
        if ( err == HA_RC_UNKNOWN_ID )
        {
            printf(" Almanac data for SV %u has not been read yet.\n", 
                    atoi(argv[3]));
            printf(" Error: %s.\n", tsync_strerror(err));
            return (1);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
            return (1);
        }
    }
    
         /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) && 
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%u\n", almanac.alm.prn);
    }
	
	printf("\n  GR (%d) Almanac: SV ID: (%u) : \n", (index), (almanac.alm.prn));
	printf("\tSV Health: %u\n", almanac.alm.svHealth);
    printf("\tWeek Num: %u\n", almanac.alm.weekNumber);	
    printf("\te: %f\n", almanac.alm.e);
    printf("\ttoa: %u\n", almanac.alm.toa);
    printf("\tio: %f\n", almanac.alm.io);
    printf("\tomegaDot: %f\n", almanac.alm.omegaDot);
    printf("\tsqrtA: %f\n", almanac.alm.sqrtA);
    printf("\tomega0: %f\n", almanac.alm.omega0);
    printf("\tomega: %f\n", almanac.alm.omega);
    printf("\tm0: %f\n", almanac.alm.m0);
    printf("\taf0: %f\n", almanac.alm.af0);
    printf("\taf1: %f\n", almanac.alm.af1);
				
		
    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
