#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    TSYNC_FixDataObj   fi;


    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: GR_GetFixData <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Fix Data message */
    index = atoi(argv[2]);

    // Send Get Fix Data transaction
    err = TSYNC_GR_getFixData(hnd, index, &fi);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

     /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) && 
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%u %f %f %f %f %u %u %u %u\n", fi.nSats,
                fi.pdop, fi.hdop, fi.vdop, fi.tdop, fi.fom, fi.tfom,
                fi.herr, fi.verr);
     }

    printf("\n");
    printf(" GR (%d) Fix Data:\n", (index));
    printf("  nSats: %u\n",        (fi.nSats));
    printf("  pdop:  %01.2f\n",     fi.pdop);
    printf("  hdop:  %01.2f\n",     fi.hdop);
    printf("  vdop:  %01.2f\n",     fi.vdop);
    printf("  tdop:  %01.2f\n",     fi.tdop);
    printf("  fom:   %u\n",        (fi.fom));
    printf("  tfom:  %u\n",        (fi.tfom));
    printf("  herr:  %u\n",        (fi.herr));
    printf("  verr:  %u\n",        (fi.verr));

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
