#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    GL_MODE            mode;
    GL_DYN             dyn;
    char               modeStr[32] = {0};
    char               dynStr[32] = {0};


    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: GR_GetMode <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Mode message */
    index = atoi(argv[2]);

    // Send Get Mode transaction
    err = TSYNC_GR_getMode(hnd, index, &mode, &dyn);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }
    
     /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) && 
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        // Dynamimcs can ONLY be printed here if the caller
        // NEVER provides just the mode. The KTSIF can't cull
        // lines from the ktsif.conf file if ONLY mode is used
        // in the set call from the library.
        printf("%d %d\n", mode, dyn);
    }

    switch (mode)
    {
        case GL_MODE_1SAT: sprintf(modeStr, "SINGLE-SAT"); break;
        case GL_MODE_STND: sprintf(modeStr, "STANDARD");   break;
        case GL_MODE_CONT: sprintf(modeStr, "CONTINUOUS"); break;
        case GL_MODE_AVRG: sprintf(modeStr, "AVERAGING");  break;
        case GL_MODE_TIME: sprintf(modeStr, "TIME-ONLY");  break;
        case GL_MODE_STBY: sprintf(modeStr, "STANDBY");    break;
        case GL_MODE_SELF: sprintf(modeStr, "SELF-TEST");  break;
        default:           sprintf(modeStr, "UNKNOWN");    break;
    }

    switch (dyn)
    {
        case GL_DYN_LAND:   sprintf(dynStr, "LAND");       break;
        case GL_DYN_SEA:    sprintf(dynStr, "SEA");        break;
        case GL_DYN_AIR:    sprintf(dynStr, "AIR");        break;
        case GL_DYN_STAT:   sprintf(dynStr, "STATIONARY"); break;
        case GL_DYN_AIR_1G: sprintf(dynStr, "AIR1G");      break;
        case GL_DYN_AIR_4G: sprintf(dynStr, "AIR4G");      break;
        default:            sprintf(dynStr, "UNKNOWN");    break;
    }

    printf("\n  GR (%d) Rcvr Mode: %s (%d) Dynamics: %s (%d)\n", (index), modeStr, (mode), dynStr, (dyn));

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
