#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    TSYNC_UtcObj      utc;

    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: GR_GetUtc <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get UTC message */
    index = atoi(argv[2]);

    // Send Get UTC
    err = TSYNC_GR_getUtc(hnd, index, &utc);

    if (err != TSYNC_SUCCESS)
    {
        // Check the error code to determine if the error is due to not having
        // data for the specified SV.
        if ( err == HA_RC_UNKNOWN_ID )
        {
            printf(" UTC data has not been read yet.\n");
            printf(" Error: %s.\n", tsync_strerror(err));
            return (1);
        }
        else
        {
            printf("  Error: %s.\n", tsync_strerror(err));
            return (1);
        }
    }
    
         /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) && 
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
		printf("%lg %g %d %g %u %u %u %d\n", utc.utc.a0, utc.utc.a1,
		    utc.utc.deltaTls, utc.utc.tot, utc.utc.wnt, utc.utc.wnlsf, 
			utc.utc.dn, utc.utc.deltaTlsf);
    }
	printf("\n  GR (%d) UTC: \n", (index));
	printf("\ta0: %lg\n", utc.utc.a0);
	printf("\ta1: %g\n", utc.utc.a1);
	printf("\tdeltaTls: %d\n", utc.utc.deltaTls);
	printf("\ttot: %g\n", utc.utc.tot);
	printf("\twnt: %u\n", utc.utc.wnt);
	printf("\twnlsf: %u\n", utc.utc.wnlsf);
	printf("\tdn: %u\n", utc.utc.dn);
	printf("\tdeltaTlsf: %d\n", utc.utc.deltaTlsf);
		
    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }


    return (ret);

} /* End - main() */
