#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle        hnd;
    int                      rc;
    int                      ret     = 0;
    int                      devIdx;
    char                    *devName = DEFAULT_DEV;
    char                     fullDevName[32];
    char                    *pgmname = argv[0];
    TSYNC_ERROR              err     = TSYNC_SUCCESS;
    int                      pgno    = 0;
    TSYNC_CapabilityPageObj  capPage;
    int                      more    = 1;
    int                      oldCai  = -1;
    int                      i;


    /* If invalid number of arguments... */
    if (argc != 2)
    {
        printf(" Usage: GetCapabilities <device index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    printf("\n");
    printf("  CAI  | IID  | Access\n");
    printf("  --------------------\n");

    while (more == 1)
    {
        // Send get capability command
        err = TSYNC_HA_getCaps(hnd, pgno, &capPage);

        if (err != TSYNC_SUCCESS)
        {
            printf("  Error: %s.\n", tsync_strerror(err));
            return (1);
        }

        // Extract capability info entries
        for (i = 0; i < TSYNC_CAPABILITIES_PER_PAGE; i++)
        {
            if (capPage.caps[i].cai == 0xFF)
            {
                break;
            }

            printf("  ");
            if (oldCai != capPage.caps[i].cai)
            {
                oldCai = capPage.caps[i].cai;
                printf("0x%02X | ", capPage.caps[i].cai);
            }
            else
            {
                printf("     | ");
            }
            printf("0x%02X | ", capPage.caps[i].iid);
            (capPage.caps[i].access == CI_ACC_BOTH) ? printf("Both\n") :
            (capPage.caps[i].access == CI_ACC_GET)  ? printf("Get\n")  :
                                                      printf("Set\n");
            }

        more = (capPage.more);
        pgno = (((pgno) + 1));
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
