#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int                 rc;
    int                 ret     = 0;
    int                 devIdx;
    char               *devName = DEFAULT_DEV;
    char                fullDevName[32];
    char               *pgmname = argv[0];
    TSYNC_ERROR         err     = TSYNC_SUCCESS;
    unsigned int        index;
    TSYNC_TimeObj       time;


    /* If invalid number of arguments... */
    if (argc != 9)
    {
        printf(" Usage: HR_SetTime <device index> <index> <year> <doy> <hour>"
               " <minute> <second> <nanoseconds>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Time message */
    index                = atoi(argv[2]);
    time.years           = atoi(argv[3]);
    time.doy             = atoi(argv[4]);
    time.hours           = atoi(argv[5]);
    time.minutes         = atoi(argv[6]);
    time.seconds         = atoi(argv[7]);
    time.ns              = atoi(argv[8]);

    printf("\n  HR (%u) Time: %u %u %u:%u:%u {%u}\n", (index), (time.years),
           (time.doy), (time.hours), (time.minutes), (time.seconds), (time.ns));

    // Send Set Format transaction
    err = TSYNC_HR_setTime(hnd, index, &time);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
