#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    OD_PIN             gpo;
    TSYNC_TimeObj      time;


    /* If invalid number of arguments... */
    if (argc != 8)
    {
        printf(" Usage: HW_SetMatchTimeLo <device index> <gpo> <doy> <hr> ");
        printf("<min> <sec> <ns>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /*Build Set Match Time Low message */
    gpo          = (OD_PIN)atoi(argv[2]);
    time.doy     = atoi(argv[3]);
    time.hours   = atoi(argv[4]);
    time.minutes = atoi(argv[5]);
    time.seconds = atoi(argv[6]);
    time.ns      = atoi(argv[7]);

    printf("\n");
    printf(" GPO %d going low at:\n", (gpo));
    printf("  Day %03u, %02u:%02u:%02u.%09u\n", time.doy, time.hours,
           time.minutes, time.seconds, time.ns);

    /* Send Set Match Time Low transaction */
    err = TSYNC_HW_setMatchTimeLo(hnd, gpo, &time);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
