#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    IL_CE              ce;


    /* If invalid number of arguments... */
    if (argc != 4)
    {
        printf(" Usage: IP_SetCodedExp <device index> <index> <ce>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Coded Expr message */
    index = atoi(argv[2]);
    ce    = (IL_CE)atoi(argv[3]); 

    printf("\n  IP (%d) Coded Expressions: (%d) ", (index), (ce));

    switch ((ce))
    {
        case IL_CE_BCDT_CF_SBS:
                printf("BCD TOY, CF, SBS\n");           break;
        case IL_CE_BCDT_CF:
                printf("BCD TOY, CF\n");                break;
        case IL_CE_BCDT:
                printf("BCD TOY\n");                    break;
        case IL_CE_BCDT_SBS:
                printf("BCD TOY, SBS\n");               break;
        case IL_CE_BCDT_BCDY_CF_SBS:
                printf("BCD TOY, BCD YR, CF, SBS\n");   break;
        case IL_CE_BCDT_BCDY_CF:
                printf("BCD TOY, BCD YR, CF\n");        break;
        case IL_CE_BCDT_BCDY:
                printf("BCD TOY, BCD YR\n");            break;
        case IL_CE_BCDT_BCDY_SBS:
                printf("BCD TOY, BCD YR, SBS\n");       break;
        case IL_CE_UNKNOWN:
        default:
                printf("Unknown\n");                    break;
    }

    // Send Set Coded Expr transaction
    err = TSYNC_IP_setCodedExpr(hnd, (IO_PORT)index, ce);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
