#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle     hnd;
    int                   rc;
    int                   ret     = 0;
    int                   devIdx;
    char                 *devName = DEFAULT_DEV;
    char                  fullDevName[32];
    char                 *pgmname = argv[0];
    TSYNC_ERROR           err     = TSYNC_SUCCESS;
    unsigned int          index;
    TSYNC_IRIGMessageObj  msg;


    /* If invalid number of arguments... */
    if (argc != 13)
    {
        printf(" Usage: IR_SetMessage <device index> <index> <sub P0> ... <sub P9>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Message message */
    index            = atoi(argv[2]);
    msg.subframes[0] = ((unsigned short)strtoul(argv[3], NULL, 16));
    msg.subframes[1] = ((unsigned short)strtoul(argv[4], NULL, 16));
    msg.subframes[2] = ((unsigned short)strtoul(argv[5], NULL, 16));
    msg.subframes[3] = ((unsigned short)strtoul(argv[6], NULL, 16));
    msg.subframes[4] = ((unsigned short)strtoul(argv[7], NULL, 16));
    msg.subframes[5] = ((unsigned short)strtoul(argv[8], NULL, 16));
    msg.subframes[6] = ((unsigned short)strtoul(argv[9], NULL, 16));
    msg.subframes[7] = ((unsigned short)strtoul(argv[10], NULL, 16));
    msg.subframes[8] = ((unsigned short)strtoul(argv[11], NULL, 16));
    msg.subframes[9] = ((unsigned short)strtoul(argv[12], NULL, 16));

    printf("\n  IR (%d) Message:\n", (index));
    printf("\n    Sub P0: %04X\n",    (msg.subframes[0]));
    printf("\n    Sub P1: %04X\n",    (msg.subframes[1]));
    printf("\n    Sub P2: %04X\n",    (msg.subframes[2]));
    printf("\n    Sub P3: %04X\n",    (msg.subframes[3]));
    printf("\n    Sub P4: %04X\n",    (msg.subframes[4]));
    printf("\n    Sub P5: %04X\n",    (msg.subframes[5]));
    printf("\n    Sub P6: %04X\n",    (msg.subframes[6]));
    printf("\n    Sub P7: %04X\n",    (msg.subframes[7]));
    printf("\n    Sub P8: %04X\n",    (msg.subframes[8]));
    printf("\n    Sub P9: %04X\n",    (msg.subframes[9]));

    // Send Set Message transaction
    err = TSYNC_IR_setMessage(hnd, index, &msg);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
