#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle       hnd;
    int                     rc;
    int                     ret     = 0;
    int                     devIdx;
    char                   *devName = DEFAULT_DEV;
    char                    fullDevName[32];
    char                   *pgmname = argv[0];
    TSYNC_ERROR             err     = TSYNC_SUCCESS;
    unsigned int            index;
    TSYNC_PTPEthernetItfObj itfInfo;


    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: PTR_GetEthernetITF <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Ethernet ITF message */
    index = atoi(argv[2]);

    // Send Get Ethernet ITF transaction
    err = TSYNC_PTR_getEthernetItf(hnd, index, &itfInfo);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) &&
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%i %i.%i.%i.%i %i.%i.%i.%i %i.%i.%i.%i\n",
                itfInfo.dhcpEnabled,
                itfInfo.staticIpAddr[0],
                itfInfo.staticIpAddr[1],
                itfInfo.staticIpAddr[2],
                itfInfo.staticIpAddr[3],
                itfInfo.netMask[0],
                itfInfo.netMask[1],
                itfInfo.netMask[2],
                itfInfo.netMask[3],
                itfInfo.defaultGateway[0],
                itfInfo.defaultGateway[1],
                itfInfo.defaultGateway[2],
                itfInfo.defaultGateway[3]);
    }

    printf("\n");
    printf(" PTR (%d) Ethernet ITF information:\n", (index));
    printf("  DHCP Enabled        : %i\n", itfInfo.dhcpEnabled);
    printf("  Static IP Address   : %i.%i.%i.%i\n", itfInfo.staticIpAddr[0],
           itfInfo.staticIpAddr[1],  itfInfo.staticIpAddr[2],  
           itfInfo.staticIpAddr[3]);
    printf("  Netmask             : %i.%i.%i.%i\n", itfInfo.netMask[0],
           itfInfo.netMask[1],    itfInfo.netMask[2],    itfInfo.netMask[3]);
    printf("  Default GW          : %i.%i.%i.%i\n", 
           itfInfo.defaultGateway[0], itfInfo.defaultGateway[1],  
           itfInfo.defaultGateway[2],  itfInfo.defaultGateway[3]);


    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
