#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle        hnd;
    int                      rc;
    int                      ret     = 0;
    int                      devIdx;
    char                    *devName = DEFAULT_DEV;
    char                     fullDevName[32];
    char                    *pgmname = argv[0];
    TSYNC_ERROR              err     = TSYNC_SUCCESS;
    unsigned int             index;
    TSYNC_PTPPortSettingsObj portSetObj;


    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: PTR_GetPortSettings <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Port Settings message */
    index = atoi(argv[2]);

    // Send Get Port Settings transaction
    err = TSYNC_PTR_getPortSettings(hnd, index, &portSetObj);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) &&
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%i %i %i %i %i %i %02X %i %i\n",
                portSetObj.portNumber,
                portSetObj.annRcptTimeout,
                portSetObj.logAnnInterval,
                portSetObj.logSyncInterval,
                portSetObj.logDelayReqInterval,
                portSetObj.logPeerDelayReqInterval,
                portSetObj.delayMechanism,
				portSetObj.syncTimeout,
				portSetObj.delayRespTimeout);
    }

    printf("\n");
    printf(" PTR (%d) Port Settings information:\n", (index));
    printf("  Port Number                     : %i\n", 
           portSetObj.portNumber);
    printf("  Ann Rcpt Timeout                : %i\n", 
           portSetObj.annRcptTimeout);
    printf("  Log Ann Interval                : %i\n", 
           portSetObj.logAnnInterval);
    printf("  Log Sync Interval               : %i\n", 
           portSetObj.logSyncInterval);
    printf("  Log Delay Request Interval      : %i\n", 
           portSetObj.logDelayReqInterval);
    printf("  Log Peer Delay Request Interval : %i\n", 
           portSetObj.logPeerDelayReqInterval);
    printf("  Delay Mechanism                 : 0x%02x\n", 
           portSetObj.delayMechanism);
	printf("  Sync Receipt Timeout            : %i\n", 
           portSetObj.syncTimeout);
	printf("  Delay-Resp Receipt Timeout      : %i\n", 
           portSetObj.delayRespTimeout);



    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
