#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle        hnd;
    int                      rc;
    int                      ret     = 0;
    int                      devIdx;
    char                    *devName = DEFAULT_DEV;
    char                     fullDevName[32];
    char                    *pgmname = argv[0];
    TSYNC_ERROR              err     = TSYNC_SUCCESS;
    unsigned int             index;
    TSYNC_PTPTimePropObj     timePropObj;


    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: PTR_GetTimeProp <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Time Prop message */
    index = atoi(argv[2]);

    // Send Get Time Prop transaction
    err = TSYNC_PTR_getTimeProperties(hnd, index, &timePropObj);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) &&
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%i %i %i %i %i %i %i %02X\n",
                timePropObj.utcOffset,
                timePropObj.utcOffsetValid,
                timePropObj.forwardLeap,
                timePropObj.backwardLeap,
                timePropObj.timeTraceable,
                timePropObj.freqTraceable,
                timePropObj.ptpTimescale,
                timePropObj.timeSource);
    }

    printf("\n");
    printf(" PTR (%d) Time Properties information:\n", (index));
    printf("  UTC Offset          : %i\n", timePropObj.utcOffset);
    printf("  UTC Offset Valid    : %i\n", timePropObj.utcOffsetValid);
    printf("  Forward Leap Sec    : %i\n", timePropObj.forwardLeap);
    printf("  Backwards Leap Sec  : %i\n", timePropObj.backwardLeap);
    printf("  Time Traceable      : %i\n", timePropObj.timeTraceable);
    printf("  Frequency Traceable : %i\n", timePropObj.freqTraceable);
    printf("  PTP Time Scale      : %i\n", timePropObj.ptpTimescale);
    printf("  Time Source         : 0x%02x\n", timePropObj.timeSource);





    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
