#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle        hnd;
    int                      rc;
    int                      ret     = 0;
    int                      devIdx;
    char                    *devName = DEFAULT_DEV;
    char                     fullDevName[32];
    char                    *pgmname = argv[0];
    TSYNC_ERROR              err     = TSYNC_SUCCESS;
    unsigned int             index;
    TSYNC_PTPUnctMasterAddObj   unctMasterAddObj;
	
	
/* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: PTR_GetUnctMasterAdd <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Port State message */
    index = atoi(argv[2]);

    // Send Get Port State transaction
    err = TSYNC_PTR_getUnctMasterAdd(hnd, index, &unctMasterAddObj);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) &&
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%02X:%02X:%02X:%02X:%02X:%02X:%02X:%02X %i %i.%i.%i.%i %i %i %i %i %i  \n",
                unctMasterAddObj.clockIdentity[0],
                unctMasterAddObj.clockIdentity[1],
                unctMasterAddObj.clockIdentity[2],
                unctMasterAddObj.clockIdentity[3],
                unctMasterAddObj.clockIdentity[4],
                unctMasterAddObj.clockIdentity[5],
                unctMasterAddObj.clockIdentity[6],
                unctMasterAddObj.clockIdentity[7],
                unctMasterAddObj.portNumber,
                unctMasterAddObj.staticIpAddr[0],
                unctMasterAddObj.staticIpAddr[1],
                unctMasterAddObj.staticIpAddr[2],
                unctMasterAddObj.staticIpAddr[3],
                unctMasterAddObj.logQuerySalveInterval,
				unctMasterAddObj.duationSlaveContracts,
				unctMasterAddObj.logAnnSlaveInterval,
				unctMasterAddObj.logSyncMasterInterval,
				unctMasterAddObj.logDelayReqSlaveInterval);
    }

    printf("\n");
    printf(" PTR (%d) Port State information:\n", (index));
    printf("  Clock Identity   : %02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x\n",
        unctMasterAddObj.clockIdentity[0],
        unctMasterAddObj.clockIdentity[1],
        unctMasterAddObj.clockIdentity[2],
        unctMasterAddObj.clockIdentity[3],
        unctMasterAddObj.clockIdentity[4],
        unctMasterAddObj.clockIdentity[5],
        unctMasterAddObj.clockIdentity[6],
        unctMasterAddObj.clockIdentity[7]);
    printf("  Port Number      : %i\n",  unctMasterAddObj.portNumber);
	printf("  Master clock Static IP address  : %i.%i.%i.%i\n",
				unctMasterAddObj.staticIpAddr[0],
                unctMasterAddObj.staticIpAddr[1],
                unctMasterAddObj.staticIpAddr[2],
                unctMasterAddObj.staticIpAddr[3]);
	printf("  The unicast query interval[slave settings]    : %i\n",  unctMasterAddObj.logQuerySalveInterval);
	printf("  The duration of the unicast contracts[slave settings]      : %i\n",  unctMasterAddObj.duationSlaveContracts);
	printf("  The unicast  announce interval[slave settings]      : %i\n",  unctMasterAddObj.logAnnSlaveInterval);
	printf("  The unicast  sync interval[master settings]      : %i\n",  unctMasterAddObj.logSyncMasterInterval);	
	printf("  The unicast  min delay-request interval[slave settings]      : %i\n",  unctMasterAddObj.logDelayReqSlaveInterval);				
				
				

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
