#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle        hnd;
    int                      rc;
    int                      ret     = 0;
    int                      devIdx;
    char                    *devName = DEFAULT_DEV;
    char                     fullDevName[32];
    char                    *pgmname = argv[0];
    TSYNC_ERROR              err     = TSYNC_SUCCESS;
    unsigned int             index;
    TSYNC_PTPUnctSlavePropObj   unctSlavePropObj;
	
	
/* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: PTR_GetUnctSlaveProp <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Port State message */
    index = atoi(argv[2]);

    // Send Get Port State transaction
    err = TSYNC_PTR_getUnctSlaveProp(hnd, index, &unctSlavePropObj);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) &&
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%i %s %i %i %i %s %i %i %i %s %i %i %i \n",
			unctSlavePropObj.negoEnabled                  ,
            unctSlavePropObj.contractAnnState             ,
			unctSlavePropObj.duationAnnContracts          ,
			unctSlavePropObj.delayAnnContracts            ,
			unctSlavePropObj.logAnnMsgInterval            ,
			unctSlavePropObj.contractSyncState            ,
			unctSlavePropObj.duationSyncContracts         ,
			unctSlavePropObj.delaySyncContracts           ,
			unctSlavePropObj.logSyncMsgInterval           ,
			unctSlavePropObj.contractDelayRespState       ,
			unctSlavePropObj.duationDelayRespContracts    ,
			unctSlavePropObj.delayDelayRespContracts      ,
			unctSlavePropObj.logDelayRespMsgInterval      );
    }

    printf("\n");

	    printf(" PTR (%d) Port State information:\n", (index));
		
		printf("  Negotiation Enabled   : %i\n"         ,
        unctSlavePropObj.negoEnabled)                  ;
	    printf("  Announce Contract State  : %s\n"      ,
        unctSlavePropObj.contractAnnState )             ;
		printf("  Announce Contract Duration  : %i\n"   ,
        unctSlavePropObj.duationAnnContracts  )         ;	
        printf("  Announce Contract Delay  : %i\n"      ,
        unctSlavePropObj.delayAnnContracts  )          ;	    
		printf("  Announce Log messgae interval  : %i\n",
        unctSlavePropObj.logAnnMsgInterval  )           ;	  
		
		printf("  Sync Contract State  : %s\n"          ,
        unctSlavePropObj.contractSyncState )            ;
		printf("  Sync Contract Duration  : %i\n"       ,
        unctSlavePropObj.duationSyncContracts  )        ;	
        printf("  Sync Contract Delay  : %i\n"          ,
        unctSlavePropObj.delaySyncContracts  )          ;	    
		printf("  Sync Log messgae interval  : %i\n"    ,
        unctSlavePropObj.logSyncMsgInterval  )          ;	  
		
		
		printf("  DelayResp Contract State  : %s\n"          ,
        unctSlavePropObj.contractDelayRespState  )           ;
		printf("  DelayResp Contract Duration  : %i\n"       ,
        unctSlavePropObj.duationDelayRespContracts  )        ;	
        printf("  DelayResp Contract Delay  : %i\n"          ,
        unctSlavePropObj.delayDelayRespContracts   )         ;	    
		printf("  DelayResp Log messgae interval  : %i\n"    ,
        unctSlavePropObj.logDelayRespMsgInterval  )          ;
			    
	
	
				
				

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
