#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle        hnd;
    int                      rc;
    int                      ret     = 0;
    int                      devIdx;
    char                    *devName = DEFAULT_DEV;
    char                     fullDevName[32];
    char                    *pgmname = argv[0];
    TSYNC_ERROR              err     = TSYNC_SUCCESS;
    unsigned int             index;
    unsigned int             mech;
    TSYNC_PTPPortSettingsObj portSetObj;


    /* If invalid number of arguments... */
    if (argc != 12)
    {
        printf(" Usage: PTR_SetPortSettings <device index> <index> <Port Number> <Ann Rct Timeout> <Log Ann Interval> <Log Sync Interval> <Log Delay Req Interval> <Log Peer Delay Req Interval> <Delay Mechanism> <Sync Timeout> <Delay-Resp Timeout>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Port Settings message */
    index = atoi(argv[2]);
    portSetObj.portNumber              = (unsigned char)atoi(argv[3]);
    portSetObj.annRcptTimeout          = (unsigned char)atoi(argv[4]);
    portSetObj.logAnnInterval          =          (char)atoi(argv[5]);
    portSetObj.logSyncInterval         =          (char)atoi(argv[6]);
    portSetObj.logDelayReqInterval     =          (char)atoi(argv[7]);
    portSetObj.logPeerDelayReqInterval =          (char)atoi(argv[8]);
    sscanf(argv[9],"%02x", &mech);
    portSetObj.delayMechanism          = (PTL_DELAY_MECH)mech;
	portSetObj.syncTimeout             = (unsigned char)atoi(argv[10]);
	portSetObj.delayRespTimeout        = (unsigned char)atoi(argv[11]);

    printf("\n");
    printf(" PTR (%d) Port Settings information:\n", (index));
    printf("  Port Number                     : %i\n", 
           portSetObj.portNumber);
    printf("  Ann Rcpt Timeout                : %i\n", 
           portSetObj.annRcptTimeout);
    printf("  Log Ann Interval                : %i\n", 
           portSetObj.logAnnInterval);
    printf("  Log Sync Interval               : %i\n", 
           portSetObj.logSyncInterval);
    printf("  Log Delay Request Interval      : %i\n", 
           portSetObj.logDelayReqInterval);
    printf("  Log Peer Delay Request Interval : %i\n", 
           portSetObj.logPeerDelayReqInterval);
    printf("  Delay Mechanism                 : 0x%02x\n", 
           portSetObj.delayMechanism);
	printf("  Sync Receipt Timeout            : %i\n", 
           portSetObj.syncTimeout);
	printf("  Delay-Resp Receipt Timeout      : %i\n", 
           portSetObj.delayRespTimeout);
		   
    // Send Set Port Settings transaction
    err = TSYNC_PTR_setPortSettings(hnd, index, &portSetObj);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }


    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
