#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle        hnd;
    int                      rc;
    int                      ret     = 0;
    int                      devIdx;
    char                    *devName = DEFAULT_DEV;
    char                     fullDevName[32];
    char                    *pgmname = argv[0];
    TSYNC_ERROR              err     = TSYNC_SUCCESS;
    unsigned int             index;
    TSYNC_PTPSyncEStatusObj  syncEItfObj;


    /* If invalid number of arguments... */
    if (argc != 7)
    {
        printf(" Usage: PTR_SetSyncEItf <device index> <index> <SyncE Enable> <Transmit Clock Enable> <ESMC Enabled > <Ssm Code>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Port Settings message */
    index = atoi(argv[2]);
    syncEItfObj.enableSyncE       = (unsigned char)atoi(argv[3]);
    syncEItfObj.xmitSyncEClock    = (unsigned char)atoi(argv[4]);
	syncEItfObj.esmcEnable        = (unsigned char)atoi(argv[5]);	
	syncEItfObj.ssmCode			  = (unsigned char)atoi(argv[6]);

    printf("\n");
    printf(" PTR (%d) Syncronous Ethernet information:\n", (index)     );
    printf("  Sync-E Enable         : %i\n", syncEItfObj.enableSyncE   );
    printf("  Transmit Sync-E Clock : %i\n", syncEItfObj.xmitSyncEClock);
	printf("  ESMC Enabled          : %i\n", syncEItfObj.esmcEnable    );
	printf("  SSM Code              : %i\n", syncEItfObj.ssmCode	   );

    // Send Set Port Settings transaction
    err = TSYNC_PTR_setSyncEItf(hnd, index, &syncEItfObj);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }


    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
