#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle        hnd;
    int                      rc;
    int                      ret     = 0;
    int                      devIdx;
    char                    *devName = DEFAULT_DEV;
    char                     fullDevName[32];
    char                    *pgmname = argv[0];
    TSYNC_ERROR              err     = TSYNC_SUCCESS;
    unsigned int             index;
    TSYNC_PTPUnctMasterAddObj   unctMasterAddObj;


    /* If invalid number of arguments... */
    if (argc != 11)
    {
        printf(" Usage: PTR_SetUnctMasterAdd <device index> <index> <Clock Identity> <Port Number> <static IP address> <Query Interval> <Duration contracts> <Log Announce Interval> <Log Sync Interval> <Delay-Request Interval>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Port Settings message */
	index = atoi(argv[2]);
    sscanf(argv[3],"%2hhx:%2hhx:%2hhx:%2hhx:%2hhx:%2hhx:%2hhx:%2hhx",
                   &(unctMasterAddObj.clockIdentity[0]),
                   &(unctMasterAddObj.clockIdentity[1]),
                   &(unctMasterAddObj.clockIdentity[2]),
                   &(unctMasterAddObj.clockIdentity[3]),
                   &(unctMasterAddObj.clockIdentity[4]),
                   &(unctMasterAddObj.clockIdentity[5]),
                   &(unctMasterAddObj.clockIdentity[6]),
                   &(unctMasterAddObj.clockIdentity[7]));

    unctMasterAddObj.portNumber          = atoi(argv[4]);
	sscanf(argv[5],"%3hhi.%3hhi.%3hhi.%3hhi",
                       &(unctMasterAddObj.staticIpAddr[0]),   &(unctMasterAddObj.staticIpAddr[1]),
                       &(unctMasterAddObj.staticIpAddr[2]),   &(unctMasterAddObj.staticIpAddr[3]));
    unctMasterAddObj.logQuerySalveInterval         =          atoi(argv[6]);
    unctMasterAddObj.duationSlaveContracts         =          atoi(argv[7]);
    unctMasterAddObj.logAnnSlaveInterval     =          atoi(argv[8]);
    unctMasterAddObj.logSyncMasterInterval =          atoi(argv[9]);
    unctMasterAddObj.logDelayReqSlaveInterval =          atoi(argv[10]);

 printf("\n");
    printf(" PTR (%d) Unicast Master clock information:\n", (index));
    printf("  Clock Identity   : %02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x\n",
        unctMasterAddObj.clockIdentity[0],
        unctMasterAddObj.clockIdentity[1],
        unctMasterAddObj.clockIdentity[2],
        unctMasterAddObj.clockIdentity[3],
        unctMasterAddObj.clockIdentity[4],
        unctMasterAddObj.clockIdentity[5],
        unctMasterAddObj.clockIdentity[6],
        unctMasterAddObj.clockIdentity[7]);
    printf("  Port Number      : %i\n",  unctMasterAddObj.portNumber);
	printf("  Master clock Static IP address  : %i.%i.%i.%i\n",
				unctMasterAddObj.staticIpAddr[0],
                unctMasterAddObj.staticIpAddr[1],
                unctMasterAddObj.staticIpAddr[2],
                unctMasterAddObj.staticIpAddr[3]);
	printf("  The unicast query interval[slave settings]    : %i\n",  unctMasterAddObj.logQuerySalveInterval);
	printf("  The duration of the unicast contracts[slave settings]      : %i\n",  unctMasterAddObj.duationSlaveContracts);
	printf("  The unicast  announce interval[slave settings]      : %i\n",  unctMasterAddObj.logAnnSlaveInterval);
	printf("  The unicast  sync interval[master settings]      : %i\n",  unctMasterAddObj.logSyncMasterInterval);	
	printf("  The unicast  min delay-request interval[slave settings]      : %i\n",  unctMasterAddObj.logDelayReqSlaveInterval);

    // Send Set Port Settings transaction
    err = TSYNC_PTR_setUnctMasterAdd(hnd, index, &unctMasterAddObj);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }


    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
