#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle       hnd;
    int                     rc;
    int                     ret     = 0;
    int                     devIdx;
    char                   *devName = DEFAULT_DEV;
    char                    fullDevName[32];
    char                   *pgmname = argv[0];
    TSYNC_ERROR             err     = TSYNC_SUCCESS;
    unsigned int            index;
    TSYNC_PTPUserDescObj    userDesc;


    /* If invalid number of arguments... */
    if (argc != 5)
    {
        printf(" Usage: PTR_SetUserDescription <device index> <index> <Device Name> <Device Location>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Module Info message */
    index = atoi(argv[2]);
    strcpy((char*)userDesc.deviceName,     argv[3]);
    strcpy((char*)userDesc.deviceLocation, argv[4]);

    // Send Get Module Info transaction
    err = TSYNC_PTR_setUserDesc(hnd, index, &userDesc);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    printf("\n");
    printf(" PTR (%d) User Description:\n", (index));
    printf("     Name    : %s\n", userDesc.deviceName);
    printf("     Location: %s\n", userDesc.deviceLocation);


    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
