#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    SIG_CTL            sc;


    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: QP_GetPpsSigCtrl <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Format message */
    index = atoi(argv[2]);

    // Send Get Format transaction
    err = TSYNC_QP_getPpsSigCtrl(hnd, (IO_PORT)index, &sc);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) && 
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%d\n", sc);
    }

    printf("\n");
    printf(" QP (%d) PPS signature control mode: ", (index));

    switch ((sc))
    {
        case SIG_CTL_NONE:
            printf("(%u) No Signature Control\n",         (sc));
            break;
        case SIG_CTL_SYNC:
            printf("(%u) Sync State Signature Control\n", (sc));
            break;
        case SIG_CTL_REF:
            printf("(%u) Ref State Signature Control\n",  (sc));
            break;
        case SIG_CTL_OFF:
            printf("(%u) Output Disabled\n",              (sc));
            break;
        default:
            printf("(%u) Unknown\n",                      (sc));
            break;
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
