#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    QL_FMT             fmt;


    /* If invalid number of arguments... */
    if (argc != 4)
    {
        printf(" Usage: QP_SetExtFormat <device index> <index> <fmt>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Format message */
    index = atoi(argv[2]);
    fmt   = (QL_FMT)atoi(argv[3]);

    printf("\n  QP (%d) Format: (%d) ", (index), (fmt));
    
    switch ((fmt))
    {
        case QL_FMT_HQ_I:       printf("HaveQuick I\n");                break;
        case QL_FMT_HQ_II:      printf("HaveQuick II\n");               break;
        case QL_FMT_HQ_IIA:     printf("HaveQuick IIA\n");              break;
        case QL_FMT_4430_STM:   printf("4430 Standard Time Message\n"); break;
        case QL_FMT_4430_XHQ:   printf("4430 Extended HaveQuick\n");    break;
        case QL_FMT_GPS_BCD:    printf("ICD-GPS-060A BCD\n");           break;
        case QL_FMT_GPS_HQ:     printf("ICD-GPS-060A HaveQuick\n");     break;
        default:                printf("Unknown\n");                    break;
    }

    // Send Set Format transaction
    err = TSYNC_QP_setExtFormat(hnd, (IO_PORT)index, fmt);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
