#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle   hnd;
    int                 rc;
    int                 ret     = 0;
    int                 devIdx;
    char               *devName = DEFAULT_DEV;
    char                fullDevName[32];
    char               *pgmname = argv[0];
    TSYNC_ERROR         err     = TSYNC_SUCCESS;
    unsigned int        index;
    TSYNC_LocalClockObj local;


    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: QR_GetLocal <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Local message */
    index = atoi(argv[2]);

    // Send Get Local transaction
    err = TSYNC_QR_getLocal(hnd, index, &local);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }
    
    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) && 
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%d %d %d %d %d %d %d %d %d %d %d\n", local.rule.ref,
                local.rule.in.month, local.rule.in.wom, local.rule.in.dow,
                local.rule.in.hour, local.rule.out.month, local.rule.out.wom,
                local.rule.out.dow, local.rule.out.hour, local.rule.offset,
                local.tz);
    }

    printf("\n  QR (%d) Time Local:\n", (index));

    printf("\n    DST Ref:        %d\n", local.rule.ref);

    printf("\n    DST IN Month:   %d\n", local.rule.in.month);

    printf("\n    DST IN WOM:     %d\n", local.rule.in.wom);

    printf("\n    DST IN DOW:     %d\n", local.rule.in.dow);

    printf("\n    DST IN Hour:    %d\n", local.rule.in.hour);

    printf("\n    DST OUT Month:  %d\n", local.rule.out.month);

    printf("\n    DST OUT WOM:    %d\n", local.rule.out.wom);

    printf("\n    DST OUT DOW:    %d\n", local.rule.out.dow);

    printf("\n    DST OUT Hour:   %d\n", local.rule.out.hour);

    printf("\n    DST OUT Offset: %d\n", local.rule.offset);

    printf("\n    TZ:             %d\n", local.tz);

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
