#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    TSYNC_ReferencePhaseDataObj entry;

    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: RS_GetPhaseData <device index> <reference id string>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }


    // Send Get Enable transaction
    err = TSYNC_RS_getReferencePhaseData(hnd, argv[2], &entry);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }
    
    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) && 
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%u %-4s %d %d %f %f %f %f %f %f\n", entry.muxPos,
                entry.source, entry.phaseValid, entry.phase, entry.phaseFiltered,
                entry.lAvg, entry.lStdDev, entry.wPhaseErr, entry.wFreqErr,
                entry.threshold);
    }

    printf("\n");
    printf("Mux:              %-u\n", (entry.muxPos));
    printf("Ref:              %-4s\n", entry.source);
    printf("Phase Valid:      %-d\n", (entry.phaseValid));
    printf("Phase:            %-d\n", (entry.phase));
    printf("Filtered Phase:   %-f\n", (entry.phaseFiltered));
    printf("Long Average:     %-f\n", (entry.lAvg));
    printf("Long Std Dev:     %-f\n", (entry.lStdDev));
    printf("Window Phase Err: %-f\n", (entry.wPhaseErr));
    printf("Window Freq Err:  %-f\n", (entry.wFreqErr));
    printf("Threshold:        %-f\n", (entry.threshold));

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
