#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle        hnd;
    int                      rc;
    int                      ret     = 0;
    int                      devIdx;
    char                    *devName = DEFAULT_DEV;
    char                     fullDevName[32];
    char                    *pgmname = argv[0];
    TSYNC_ERROR              err     = TSYNC_SUCCESS;
    TSYNC_TableTypeObj       type;
    TSYNC_ReferenceTableObj  tbl;
    unsigned int             i;


    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: RS_GetTable <device index> <table type>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Table message */
    type.type = (RS_TABLE_TYPE)atoi(argv[2]);

    // Send Get Table transaction
    err = TSYNC_RS_getTable(hnd, &type, &tbl);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    printf("\n");
    printf("  Idx | En  | Pri | Time | 1PPS\n");
    printf("  -----------------------------\n");
    for (i = 0; i < TSYNC_TABLE_ENTRY_NUM; i++)
    {
        printf("  ");
        printf("%-3d | ", i);
        ((tbl.rows[i].enab) == 1) ? printf("EN  | ") : printf("DIS | ");
        printf("%-3d | ", (tbl.rows[i].prio));
        printf("%-4s | ", tbl.rows[i].time);
        printf("%-4s\n", tbl.rows[i].pps);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
