#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle          hnd;
    int                        rc;
    int                        ret     = 0;
    int                        devIdx;
    char                      *devName = DEFAULT_DEV;
    char                       fullDevName[32];
    char                      *pgmname = argv[0];
    TSYNC_ERROR                err     = TSYNC_SUCCESS;
    TSYNC_OscCustomMessageObj  cm;
    int                        i;


    /* If invalid number of arguments... */
    if (argc < 3)
    {
        printf(" Usage: XO_SendMessage <device index> <cmd>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* clear message buffer */
    memset(cm.msg, 0, sizeof(cm.msg));

    /* Build Send Custom Message message */
    for (i = 2; i < argc; i++)
    {
        /* Check message length */
        if ((strlen(cm.msg) + strlen(argv[i]) + 2) > sizeof(cm.msg))
        {
            printf("Custom message too large.\n");
            return (1);
        }

        strcat(cm.msg, argv[i]);
        strcat(cm.msg, " ");
    }
    cm.msg[strlen(cm.msg) - 1] = '\r';
    cm.msg[strlen(cm.msg)]     = '\n';
    cm.len = strlen(cm.msg);

    printf("\n");
    printf(" XO Custom Msg Sent:\n");
    printf("  ------------------------\n");
    printf("  %s\n", (cm.msg));
    printf("\n");

    // Send Send Custom Message transaction
    err = TSYNC_XO_setMessage(hnd, &cm);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }
    printf("  Response:\n");
    printf("  ------------------------\n");
    printf("  %s\n", (cm.msg));
    printf("\n");

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
