#include <sys/ioctl.h>
#include <sys/file.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include <ddtpro.h>
#include <tpro.h>
#include <string.h>

#include "tpro_macros.h"
#include "tpro_capabilities.h"
#include "tsync.h"

/*==========================================================================
  TPRO_getFPGA
==========================================================================*/

unsigned char TPRO_getFPGA (TPRO_BoardObj *hnd, char *fpga)
{
    tpro_boolean_t boardHasFPGA;

    unsigned char numericalVersion[2];

    /**
    *** verify valid string buffer
    **/
    CHECK_HANDLE(hnd);
    CHECK_NOT_NULL(fpga);
    
  /**
  ***  determine the board type and emulate legacy
  ***  call if needed
  **/

  #include "tpro_legacy_getfpga.h"
  
    CHECK(TPRO_boardHasFPGA(hnd, &boardHasFPGA), TPRO_SUCCESS);
    if (boardHasFPGA != TPRO_TRUE) {
        return (TPRO_INVALID_BOARD_TYPE_ERR);
    }
    
    if (ioctl (hnd->file_descriptor, IOCTL_TPRO_GET_FPGA, numericalVersion) < 0) {
        return (TPRO_COMM_ERR);
    }

    /* convert from the two byte values to the string form of the result */
    sprintf(fpga,"%02u.%02u",numericalVersion[0],numericalVersion[1]);

  return (TPRO_SUCCESS);
}
