#include <sys/ioctl.h>
#include <stdio.h>
#include <tpro.h>
#include <ddtpro.h>

#include "tpro_macros.h"
#include "tpro_capabilities.h"
#include "tsync.h"

/*==========================================================================
  TPRO_getLatitude
==========================================================================*/

unsigned char TPRO_getLatitude (TPRO_BoardObj *hnd, TPRO_LatObj *Latp)
{
  int err;

  LatObj Latitude;

  tpro_boolean_t boardSupportsGPS;

  // make sure we have valid handle
  CHECK_HANDLE(hnd);
  CHECK_NOT_NULL(Latp);

  /**
  ***  determine the board type and emulate legacy
  ***  call if needed
  **/

  #define LATOBJ_NAME Latp
  #include "tpro_legacy_getlatitude.h"
  #undef LATOBJ_NAME

  CHECK(TPRO_boardSupportsGPSHardware(hnd, &boardSupportsGPS), TPRO_SUCCESS);
  if (boardSupportsGPS != TPRO_TRUE) {
      return (TPRO_INVALID_BOARD_TYPE_ERR);
  }
  
  /**
  ***  create I/O control returns nonzero on success
  **/

  err = ioctl(hnd->file_descriptor, IOCTL_TPRO_GET_LATITUDE, &Latitude);
  if (err < 0)	return (TPRO_COMM_ERR);

  /**
  ***  fill-in users context
  **/
  {
    float temp;

    temp = (float) (Latitude.minutes);
    temp /= 10000.0;
    Latp->degrees = Latitude.degrees;
    Latp->minutes = temp;
  }

  return (TPRO_SUCCESS);
}
