#include <sys/ioctl.h>
#include <stdio.h>
#include <tpro.h>
#include <ddtpro.h>

#include "tpro_macros.h"
#include "tpro_capabilities.h"
#include "tsync.h"

// heartbeat constants
#define OPTION_HI_RATE		(0)		// hi oscillator option
#define OPTION_LO_RATE  	(1)		// low oscillator option


/*==========================================================================
  TPRO_setHeartbeat
==========================================================================*/

unsigned char TPRO_setHeartbeat (TPRO_BoardObj *hnd, TPRO_HeartObj *Heartp)
{
  unsigned char option;
  int err;

  HeartObj Heartbeat;

  tpro_boolean_t boardSupportHighRate;

  // make sure we have valid handle
  CHECK_HANDLE(hnd);
  CHECK_NOT_NULL(Heartp);

  /**
  ***  determine the board type and emulate legacy
  ***  call if needed
  **/

  #include "tpro_legacy_setheartbeat.h"
  
  /**
  ***  determine how to set heartbeat based on board options
  **/

  CHECK(TPRO_boardSupportsHighRateHeartbeat(hnd, &boardSupportHighRate),
        TPRO_SUCCESS);

  if (boardSupportHighRate == TPRO_TRUE) {
    option = OPTION_LO_RATE;
  }
  else {
    option = OPTION_HI_RATE;
  }


  /**
  ***  calculate divide number
  **/

  if (option == OPTION_HI_RATE) {

    // calculate
    Heartbeat.divNumber = (unsigned short) (3000000 / Heartp->frequency);

    // verify valid divide number
    if ( (Heartp->frequency > 1000000) || (Heartp->frequency < 45.7771) ) return (TPRO_FREQ_ERR);
    if (Heartbeat.divNumber % 3) return (TPRO_FREQ_ERR);
  }

  else {

    // calculate
    Heartbeat.divNumber = (unsigned short) (1000 / Heartp->frequency);

    // verify valid divide number
    if ( (Heartp->frequency > 500) || (Heartp->frequency < 0.0152593) ) return (TPRO_FREQ_ERR);
    if (Heartbeat.divNumber < 2) return (TPRO_FREQ_ERR);
  }


  /**
  ***  fill-in working context
  **/

  Heartbeat.signalType = Heartp->signalType;
  Heartbeat.outputType = Heartp->outputType;


  /**
  ***  create I/O control
  **/

  err = ioctl(hnd->file_descriptor, IOCTL_TPRO_SET_HEARTBEAT, &Heartbeat);
  err = (err < 0) ? TPRO_COMM_ERR : TPRO_SUCCESS;

  return (err);
}
