#include <sys/ioctl.h>
#include <stdio.h>
#include <tpro.h>
#include <ddtpro.h>


#include "tpro_macros.h"
#include "tpro_capabilities.h"
#include "tsync.h"

/*==========================================================================
  TPRO_setYear
==========================================================================*/

unsigned char TPRO_setYear (TPRO_BoardObj *hnd, unsigned short *yr)
{
  int err;

  tpro_boolean_t boardSupportsGregorianDate;

  // make sure we have valid handle
  CHECK_HANDLE(hnd);
  CHECK_NOT_NULL(yr);

  /**
  ***  determine the board type and emulate legacy
  ***  call if needed
  **/

  #define YEAR_NAME yr
  #include "tpro_legacy_setyear.h"
  #undef YEAR_NAME
  
  CHECK(TPRO_boardSupportsGregorianDate(hnd, &boardSupportsGregorianDate), TPRO_SUCCESS);
  if (boardSupportsGregorianDate != TPRO_TRUE) {
      return (TPRO_INVALID_BOARD_TYPE_ERR);
  }

  /**
  ***  check for valid year
  **/

  if ( (*yr < 1990) || (*yr > 2999) )
    return (TPRO_YEAR_PARM_ERR);


  /**
  ***  create I/O control returns nonzero on success
  **/

  err = ioctl(hnd->file_descriptor, IOCTL_TPRO_SET_YEAR, yr);
  err = (err < 0) ? TPRO_COMM_ERR : TPRO_SUCCESS;

  return (err);
}
