#include "example.h"

void printUsage(const char *pgmname);
#define OPT_ARG_LIST        "hd:i:"

int main (int argc, char *argv[])
{
    const int      MAX_DEV_NAME_LENGTH = 1024;
    char           deviceName[MAX_DEV_NAME_LENGTH];
    char           *devName = DEFAULT_DEV;
    int            devIdx = DEFAULT_DEV_NUM;

    int            retVal;
    TPRO_BoardObj *pBoard;
    TPRO_AltObj    tproAltitude;
    unsigned char  rv;
    
    int             optRc;
    char           *pgmname = argv[0];

    retVal = EXIT_SUCCESS;

    /***************************************************
     * Parse command line
     */
    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 ) {
        switch ( optRc ) {
        case 'd' :
            devName = optarg;
            break;
        case 'i' :
            devIdx = atoi(optarg);
            break;

        case 'h' :
        default:
            printUsage( pgmname );
            return EXIT_FAILURE;
        }
    }

    if ( optind != argc ) {
        printUsage( pgmname );
        return EXIT_FAILURE;
    }

    /* Construct the TSAT device name */
    snprintf(deviceName, MAX_DEV_NAME_LENGTH, "%s%s%d", DEVICE_ROOT, devName, devIdx);
    
    /* Open the indexed TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    
    /* If unable to open the TSAT device... */
    if (rv != TPRO_SUCCESS) {
        printf (" TPRO_open(%s) error: %s\n", deviceName, tpro_strerror(rv));
        return (EXIT_FAILURE);
    }

    /* Wait until this handle is the first user of the device. */
    if (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS) 
    {
        printf(" Waiting to become first user...\n");
        while (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS);
    }

    /* Get the TSAT altitude */
    rv = TPRO_getAltitude(pBoard, &tproAltitude);
    if (rv != TPRO_SUCCESS) 
    {
        printf(" TPRO_getAltitude Error: %s\n", tpro_strerror(rv));
        retVal = EXIT_FAILURE;
    }
    else
    {
        /* Display the TSAT altitude */
        printf(" Device Name: %s\n", deviceName);
        printf(" Altitude: %f\n", tproAltitude.meters);
    }
    
    /* Close the TSAT device */
    rv = TPRO_close(pBoard);
    if (rv != TPRO_SUCCESS) 
    {
        printf(" TPRO_close Error: %s\n", tpro_strerror(rv));
        retVal = EXIT_FAILURE;
    }

    return (retVal);

} /* End - main() */

void printUsage(const char *pgmname) {
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmname );
    (void)printf("\t-h   help message\n");
    (void)printf("\n");
    (void)printf("\t-d   device name  (default: %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (default: %d)\n", DEFAULT_DEV_NUM);
}    
