#include "example.h"

#include <time.h>
#include <locale.h>
#include <langinfo.h>

int main (int argc, char *argv[])
{
    const int MAX_DEV_NAME_LENGTH = 1024;
    const char *pgmname = argv[0];

    char           deviceName[MAX_DEV_NAME_LENGTH];
    int            retVal;
    unsigned char  rv;

    TPRO_BoardObj *pBoard;
    TPRO_DateObj   tproDate;

    retVal = EXIT_SUCCESS;
    
    /* If invalid number of arguments */
    if (argc != 2) 
    {
        fprintf (stderr, "Usage: GetDate <device index>\n");
        return EXIT_FAILURE;
    }
    
    /* Construct the TSAT/TPRO device name */
    snprintf(deviceName, MAX_DEV_NAME_LENGTH, "%s%s%d", DEVICE_ROOT, DEFAULT_DEV, atoi(argv[1]));
    
    /* Open the indexed TPRO/TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    if (rv != TPRO_SUCCESS) {
        fprintf (stderr, "%s: TPRO_open(%s) failed: %s\n",
                 pgmname, deviceName, tpro_strerror(rv));
        return EXIT_FAILURE;
    }

    /* Wait until this handle is the first user of the device. */
    if (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS) {
        printf(" Waiting to become first user...\n");
        while (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS);
    }

    /* Get the current TPRO/TSAT date */
    rv = TPRO_getDate (pBoard, &tproDate);
    if (rv != TPRO_SUCCESS) {
        fprintf (stderr, "%s: TPRO_getDate(%s) failed: %s\n", pgmname, deviceName,
                 tpro_strerror(rv));    
        retVal = EXIT_FAILURE;
    }
    else {
        char dateString[MAX_DEV_NAME_LENGTH];
        struct tm time;

        time.tm_mday = tproDate.day;
        time.tm_mon = tproDate.month - 1;
        time.tm_year = tproDate.year - 1900;
        
        setlocale(LC_ALL, "");
        strftime(dateString, sizeof(dateString), nl_langinfo(D_FMT), &time);

        /* Display the current TPRO/TSAT date */
        printf("Device Name: %s\n", deviceName);
        printf("Year:    %4u\n", tproDate.year);
        printf("Month:   %4u\n", tproDate.month);
        printf("Date:    %4u\n", tproDate.day);
        printf("Locally: %s (locale: %s)\n", dateString, setlocale(LC_TIME,NULL));
    }
    
    /* Close the TPRO/TSAT device */
    rv = TPRO_close (pBoard);

    /* If unable to open the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        fprintf (stderr, "%s: TPRO_close(%s) failed: %s\n", pgmname, deviceName, 
                 tpro_strerror(rv));
        retVal = EXIT_FAILURE;
    }

    return (retVal);
    
} /* End - main() */
