#include "example.h"

int main (int argc, char *argv[])
{
    char           deviceName[32];
    int            retVal;
    TPRO_BoardObj *pBoard;
    TPRO_WaitObj   WaitObj;
    unsigned char  rv;
    unsigned int   count;

    retVal = 0;

    /* If invalid number of arguments... */
    if (argc != 2)
    {
        printf(" Usage: GetEvent <device index>\n");
        return (1);
    }
    
    /* Construct the TPRO/TSAT device name */
    sprintf(deviceName, "%s%s%d", DEVICE_ROOT, DEFAULT_DEV, atoi(argv[1]));
    
    /* Open the TPRO/TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    
    /* If unable to open the TRPO/TSAT device */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Open '%s'!! [%d]\n", deviceName, rv);
        return (1);
    }

    /* Wait until this handle is the first user of the device. */
    if (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS) 
    {
        printf(" Waiting to become first user...\n");
        while (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS);
    }

    printf(" Device Name: %s\n", deviceName);

    /* Set timeout in jiffies (10 seconds) */
    WaitObj.jiffies = 10 * 100;

    /*
    **  simulate 3 events and read times
    */
    TPRO_simEvent(pBoard);
    sleep (1);
    TPRO_simEvent(pBoard);
    sleep (1);
    TPRO_simEvent(pBoard);

    for (count = 0; count < 4; count++) {

        printf (" Waiting for event %u .......\n", count);

        /* Wait for TPRO/TSAT device event */
        rv = TPRO_waitEvent(pBoard, &WaitObj);
        
        switch (rv) {
        case TPRO_SUCCESS:
            printf(" Received Interrupt\n");
            printf("   Days: %d\n", WaitObj.days);
            printf("   Hours: %d\n", WaitObj.hours);

            printf("   Minutes: %d\n", WaitObj.minutes);
            printf("   Seconds: %f\n", WaitObj.seconds);
            break;
            
        case TPRO_TIMEOUT_ERR:
            printf(" Timeout Waiting for Time-tagged Event!!\n\n");
            break;
            
        case TPRO_COMM_ERR:
            printf(" Error Communicating With Timing Board!!\n\n");
            break;
            
        default:
            printf("Error: %s\n\n", tpro_strerror(rv));
        }

    }

    /* Close the TPRO/TSAT device */
    rv = TPRO_close (pBoard);
    
    /* If unable to close the TPRO/TSAT device */
    if (rv != TPRO_SUCCESS) {
        printf (" Could Not Close Board!! [%d]\n", rv);
        retVal = 1;
    }

    return (retVal);

} /* End - main() */
