#include "example.h"

int main (int argc, char *argv[])
{
    char           deviceName[32];
    int            retVal;
    TPRO_BoardObj *pBoard;
    char           driver [7];
    char           firmware [5];
    unsigned char  rv;

    retVal = 0;
    
    /* If invalid number of events... */
    if (argc != 2) 
    {
        printf(" Usage: GetInfo <device index>\n");
        return (1);
    }
    
    /* Construct the TPRO/TSAT device name */
    sprintf(deviceName, "%s%s%d", DEVICE_ROOT, DEFAULT_DEV, atoi(argv[1]));
    
    /* Open the TPRO/TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    
    /* If unable to open the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf (" Could Not Open '%s'!! [%d]\n", deviceName, rv);
        return (1);
    }
    else
    {
        /* Display the TPRO/TSAT board options */
        printf(" Device Name: %s\n", deviceName);
        printf(" Options: 0x%04X\n", pBoard->options);
    }

    /* Wait until this handle is the first user of the device. */
    if (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS) 
    {
        printf(" Waiting to become first user...\n");
        while (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS);
    }

    /* Get the TPRO/TSAT driver revision */
    rv = TPRO_getDriver(pBoard, driver);
    
    /* If unable to retrieve the TPRO/TSAT driver revision... */
    if (rv != TPRO_SUCCESS) 
    {
        printf("Could not retrieve driver version, error: %d\n", rv);
        retVal = 1;
    }
    
    /* Display the driver revision */
    printf ("  Driver Version: %s\n", driver);

    /* Get the TPRO/TSAT firmware revision */
    rv = TPRO_getFirmware(pBoard, firmware);
    
    /* If unable to get the TPRO/TSAT firmware revision */
    if (rv != TPRO_SUCCESS) 
    {
        printf("Could not retrieve firmare version, error: %d\n", rv);
        retVal = 1;
    }
    
    /* Display the TPRO/TSAT firmware revision */
    printf("  Firmware: %s\n", firmware);

    /* Close the TPRO/TSAT device */
    rv = TPRO_close (pBoard);
    
    /* If unable to close the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf("Could Not Close Board!! [%d]\n", rv);
        retVal = 1;
    }

    return (retVal);

} /* End - main() */
