#include "example.h"

int main (int argc, char *argv[])
{
    char           deviceName[32];
    int            retVal;
    TPRO_BoardObj *pBoard;
    TPRO_LatObj    tproLatitude;
    unsigned char  rv;

    retVal = 0;
    
    /* If invalid number of arguments... */
    if (argc != 2) 
    {
        printf(" Usage: GetLatitude <device index>\n");
        return (1);
    }
    
    /* Construct the TSAT device name */
    sprintf(deviceName, "%s%s%d", DEVICE_ROOT, DEFAULT_DEV, atoi(argv[1]));
    
    /* Open the TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    
    if(rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Open '%s'!! [%d]\n", deviceName, rv);
        return (1);
    }

    /* Wait until this handle is the first user of the device. */
    if (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS) 
    {
        printf(" Waiting to become first user...\n");
        while (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS);
    }

    /* Get the TSAT current latitude */
    rv = TPRO_getLatitude(pBoard, &tproLatitude);
    
    if (rv != TPRO_SUCCESS) 
    {
        switch (rv) {
        
        case TPRO_INVALID_BOARD_TYPE_ERR:
            printf(" Error: Incorrect device type.\n");
            break;
        case TPRO_COMM_ERR:
            printf(" Error: Communication.\n");
            break;
        default:
            printf(" Could not retrieve latitude!!!\n");    
        } /* end - switch(rv) */
        
        retVal = 1;
    }
    else
    {
        /* Display the TSAT current latitude */
        printf(" Device Name: %s\n", deviceName);
        printf(" degrees: %d\n", tproLatitude.degrees);
        printf(" minutes: %f\n", tproLatitude.minutes);
    }

    /* Close the TSAT device */
    rv = TPRO_close (pBoard);
    
    /* If unable to close the TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Close Board!! [%d]\n", rv);
        retVal = 1;
    }

    return (retVal);

} /* End - main() */
