#include "example.h"

int main (int argc, char *argv[])
{

    const int      MAX_DEV_NAME_LENGTH = 1024;
    char           deviceName[MAX_DEV_NAME_LENGTH];
    char           *pgmname = argv[0];

    int            retVal;
    TPRO_BoardObj *pBoard;
    TPRO_TimeObj   tproTime;
    unsigned char  rv;

    retVal = EXIT_SUCCESS;

    /* If invalid number of arguments... */
    if (argc != 2)
    {
        printf(" Usage: GetTime <device index>\n");
        return EXIT_FAILURE;
    }

    /* Construct the TPRO/TSAT device name */
    snprintf(deviceName, MAX_DEV_NAME_LENGTH, "%s%s%d", DEVICE_ROOT, DEFAULT_DEV, atoi(argv[1]));

    /* Open the TPRO/TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    if (rv != TPRO_SUCCESS) {
        fprintf (stderr, "%s: TPRO_open(%s) failed: %s\n", pgmname, deviceName,
                 tpro_strerror(rv));
        return EXIT_FAILURE;
    }

    /* Get the TPRO/TSAT current time */
    rv = TPRO_getTime(pBoard, &tproTime);
    if (rv != TPRO_SUCCESS) {
        printf ("%s: TPRO_getTime failed: %s\n", pgmname, tpro_strerror(rv));
        retVal = EXIT_FAILURE;
    }
    else
    {
        /* Display the TPRO/TSAT current time */
        printf (" Device Name: %s\n", deviceName);
        printf (" Days:        %d\n", tproTime.days);
        printf (" Hours:       %d\n", tproTime.hours);
        printf (" Minutes:     %d\n", tproTime.minutes);
        printf (" Seconds:     %f\n", tproTime.secsDouble);
        printf (" TCODE:       %d\n", (tproTime.flags & 0x2) ? 1 : 0);
        printf (" SYNC:        %d\n", (tproTime.flags & 0x4) ? 1 : 0);
    }

    /* Close the TPRO/TSAT device */
    rv = TPRO_close(pBoard);
    if (rv != TPRO_SUCCESS) {
        printf ("%s: TPRO_close() failed: %s\n", pgmname, tpro_strerror(rv));
        retVal = EXIT_FAILURE;
    }

    return (retVal);

} /* End - main() */
