#include "example.h"

int main (int argc, char *argv[])
{
    char           deviceName[32];
    int            retVal;
    TPRO_BoardObj *pBoard;
    TPRO_MemObj    Mem;
    unsigned char  rv;

    retVal = 0;
    
    /* If invalid number of arguments... */
    if (argc != 3) 
    {
        printf(" Usage: Peek <device index> <offset in hex>\n");
        return (1);
    }

    /* Construct the TSAT/TPRO device name */
    sprintf(deviceName, "%s%s%d", DEVICE_ROOT, DEFAULT_DEV, atoi(argv[1]));
    
    Mem.offset = (unsigned short) strtoul (argv[2], NULL, 16);

    /* Open the TPRO/TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    
    /* If unable to open the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Open Board '%s'!! [%d]\n", deviceName, rv);
        printf(" Options: 0x%04X\n", pBoard->options);
        return (1);
    }

    /* Perform the TPRO/TSAT peek operation */
    rv = TPRO_peek(pBoard, &Mem);
    
    /* If unable to perform the peek operation... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could not perform peek operation, error: %d\n", rv);
        retVal = 1;
    }
    else 
    {
        printf(" Value: 0x%08lX\n", Mem.l_value);
    }

    /* Close the TPRO/TSAT device */
    rv = TPRO_close(pBoard);
    
    /* If unable to close the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Close Board!! [%d]\n", rv);
        retVal = 1;
    }

    return (retVal);

} /* End - main() */
