#include "example.h"

int main (int argc, char *argv[])
{
    char           deviceName[32];
    int            retVal;
    TPRO_BoardObj *pBoard;
    TPRO_MemObj    Mem;
    unsigned char rv;

    retVal = 0;
    
    /* If invalid number of arguments... */
    if (argc != 4) 
    {
        printf ("Error program usage: Poke <device index>"
                " <offset in hex> <value in hex>\n");
        return (1);
    }
    
    /* Construct the TSAT/TPRO device name */
    sprintf(deviceName, "%s%s%d", DEVICE_ROOT, DEFAULT_DEV, atoi(argv[1]));
    
    Mem.offset = (unsigned short) strtoul (argv[2], NULL, 16);
    Mem.value  = (unsigned short) strtoul (argv[3], NULL, 16);
    Mem.l_value  = (unsigned int) strtoul (argv[3], NULL, 16);

    /* Open the TPRO/TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    
    /* If unable to open the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf ("Could Not Open Board!! [%d]\n", rv);
        printf ("Options: 0x%04X\n", pBoard->options);
        return (1);
    }
    
    /* Perform the TPRO/TSAT poke operation */
    rv = TPRO_poke(pBoard, &Mem);
    
    /* If unable to perform the TPRO/TSAT poke operation... */ 
    if (rv != TPRO_SUCCESS) 
    {
        printf ("Could not perform poke operation, error: %d\n", rv);
        retVal = 1;
    }

    /* Close the TPRO/TSAT device */
    rv = TPRO_close(pBoard);
    
    /* If unable to close the TPRO/TSAT device */
    if (rv != TPRO_SUCCESS) 
    {
        printf ("Could Not Close Board!! [%d]\n", rv);
        retVal = 1;
    }

    return (retVal);
    
} /* End - main() */
