#include "example.h"

int main (int argc, char *argv[])
{
    char           deviceName[32];
    int            microseconds;
    int            retVal;
    TPRO_BoardObj *pBoard;
    unsigned char  rv;
    
    retVal = 0;

    /* If invalid number of arguments... */
    if (argc != 3) 
    {
        printf(" Usage: SetPropDelay <device index> <delay in microseconds>\n");
        return (1);
    }

    /* Construct the TPRO/TSAT device name */
    sprintf(deviceName, "%s%s%d", DEVICE_ROOT, DEFAULT_DEV, atoi(argv[1]));
    
    /* Read delay in microseconds argument */
    sscanf (argv[2], "%10d", &microseconds);
    
    /* Open the TPRO/TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    
    /* If unable to open the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Open Board '%s'!! [%d]\n", deviceName, rv);
        return (1);
    }

    /* Wait until this handle is the first user of the device. */
    if (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS) 
    {
        printf(" Waiting to become first user...\n");
        while (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS);
    }

    /* Set the TPRO/TSAT propagation delay */
    rv = TPRO_setPropDelayCorr(pBoard, &microseconds);
    
    /* If unable to set the TPRO/TSAT propagation delay... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Set Delay Factor!! [%d]\n", rv);
        retVal = 1;
    }

    /* Close the TPRO/TSAT device */
    rv = TPRO_close(pBoard);
    
    /* If unable to close the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Close Board!! [%d]\n", rv);
        retVal = 1;
    }

    return (retVal);

} /* End - main() */
