#include "example.h"

int main (int argc, char *argv[])
{
    char           deviceName[32];
    int            retVal;
    int            tmp;
    TPRO_BoardObj *pBoard;
    TPRO_TimeObj   tproTime = {0};
    unsigned char  rv;

    retVal = 0;
    
    /* If invalid number of arguments... */
    if (argc != 6)
    {
        printf(" Usage: SetTime <device index> <days> <hours> <minutes> "
               "<seconds>\n");
        return (1);
    }

    /* Construct the TPRO/TSAT device name */
    sprintf(deviceName, "%s%s%d", DEVICE_ROOT, DEFAULT_DEV, atoi(argv[1]));
    
    /*
    **  read into time object
    */
    sscanf (argv[2], "%10d", &tmp);
    tproTime.days = (unsigned short) tmp;

    sscanf (argv[3], "%10d", &tmp);
    tproTime.hours = (unsigned char) tmp;

    sscanf (argv[4], "%10d", &tmp);
    tproTime.minutes = (unsigned char) tmp;

    sscanf (argv[5], "%10d", &tmp);
    tproTime.seconds = (unsigned char) tmp;

    /* Open the TPRO/TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    
    /* If unable to open the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Open Board '%s'!! [%d]\n", deviceName, rv);
        return (1);
    }

    /* Wait until this handle is the first user of the device. */
    if (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS) 
    {
        printf(" Waiting to become first user...\n");
        while (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS);
    }

    /* Set the TPRO/TSAT current time */
    rv = TPRO_setTime(pBoard, &tproTime);
    
    /* If unable to set the TPRO/TSAT current time... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could not set time!! [%d]\n", rv);
        retVal = 1;
    }

    /* Close the TPRO/TSAT device */
    rv = TPRO_close(pBoard);
    
    /* If unable to close the TPRO/TSAT device */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Close Board!! [%d]\n", rv);
        retVal = 1;
    }

    return (retVal);

} /* End - main() */
