#include "example.h"

int main (int argc, char *argv[])
{
    char            deviceName[32];
    int             retVal;
    TPRO_BoardObj  *pBoard;
    unsigned char   rv;
    unsigned short  year;
    
    retVal = 0;

    /* If invalid number of arguments... */
    if (argc != 3) 
    {
        printf(" Usage: SetYear <device index> <year>\n");
        return (1);
    }

    /* Construct the TPRO/TSAT device name */
    sprintf(deviceName, "%s%s%d", DEVICE_ROOT, DEFAULT_DEV, atoi(argv[1]));
    
    /* Read the year argument */
    sscanf (argv[2], "%5hd", &year);
    
    /* Open the TPRO/TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    
    /* If unable to open the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Open Board '%s'!! [%d]\n", deviceName, rv);
        return (1);
    }

    /* Wait until this handle is the first user of the device. */
    if (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS) 
    {
        printf(" Waiting to become first user...\n");
        while (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS);
    }

    /* Set the TPRO/TSAT current year */
    rv = TPRO_setYear(pBoard, &year);
    
    /* If unable to set the current TPRO/TSAT year... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Set Year!! [%d]\n", rv);
        retVal = 1;
    }

    /* Close the TPRO/TSAT device */
    rv = TPRO_close(pBoard);
    
    /* If unable to close the TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Close Board!! [%d]\n", rv);
        retVal = 1;
    }

    return (retVal);
    
} /* End - main() */
