#include "example.h"

int main (int argc, char *argv[])
{
    char           deviceName[32];
    int            retVal;
    TPRO_BoardObj *pBoard;
    unsigned char  rv;

    retVal = EXIT_SUCCESS;

    /* If invalid number of arguments... */
    if (argc != 2) 
    {
      printf(" Usage: SimEvent <device index>\n");
      return EXIT_FAILURE;
    }

    /* Construct the TPRO/TSAT device name */
    sprintf(deviceName, "%s%s%d", DEVICE_ROOT, DEFAULT_DEV, atoi(argv[1]));
    
    /* Open the TPRO/TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    
    /* If unable to open TPRO/TSAT device... */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Open Board '%s'!! [%d]\n", deviceName, rv);
        return EXIT_FAILURE;
    }

    /* Wait until this handle is the first user of the device. */
    if (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS) 
    {
        printf(" Waiting to become first user...\n");
        while (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS);
    }

    /* Simulate external event to the TPRO/TSAT device */
    rv = TPRO_simEvent (pBoard);
    
    /* If unable to simulate the TPRO/TSAT event */
    if (rv != TPRO_SUCCESS)
    {
        printf(" Could Not Simulate Event: %s \n", tpro_strerror(rv));
        retVal = EXIT_FAILURE;
    }

    /* Close the TPRO/TSAT device */
    rv = TPRO_close(pBoard);
    
    /* If unable to close the TPRO/TSAT device */
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Could Not Close Board!! [%d]\n", rv);
        retVal = EXIT_FAILURE;
    }

    return (retVal);

} /* End - main() */
