#include "example.h"

#define SPACER "    "

static unsigned char DisplayPosData (TPRO_BoardObj *hnd);
static unsigned char TestEvent      (TPRO_BoardObj *hnd);
static unsigned char TestHeartbeat  (TPRO_BoardObj *hnd, int timeout);
static unsigned char TestMatch      (TPRO_BoardObj *hnd);

int main (int argc, char *argv[])
{
    char           deviceName[32];
    char           driver[32];
    TPRO_BoardObj *pBoard;
    TPRO_DateObj   date;
    /* TODO: TPRO_MemObj    memObj; */
    TPRO_TimeObj   time;
    unsigned char  rv;
    unsigned char  status;
    unsigned char  syncEnable;
    unsigned int   count;

    status = 0;
    
    /* If invalid number of arguments... */
    if (argc != 2) 
    {
        printf(" Usage: GetTime <device index>\n");
        return (1);
    }
    
    /* Construct the TPRO/TSAT device name */
    sprintf(deviceName, "%s%s%d", DEVICE_ROOT, DEFAULT_DEV, atoi(argv[1]));
    
    /* Open the TPRO/TSAT device */
    rv = TPRO_open(&pBoard, deviceName);

    if (rv != TPRO_SUCCESS) {
        printf ("Could Not Open Board!! [%d]\n", rv);
        return (1);
    }

    /* Retrieve the driver revision */
    rv = TPRO_getDriver (pBoard, driver);
    
    if (rv != TPRO_SUCCESS) {
        printf ("Could not retrieve driver version, error: %d\n", rv);
        return (1);
    }
    
    printf("\n------------------------------------------------------------\n");
    printf ("Driver Version: %s\n", driver);
    printf("------------------------------------------------------------\n");

    printf("\nReset Firmware: ");
    fflush(stdout);
    
    rv = TPRO_resetFirmware(pBoard);
    
    if (rv != TPRO_SUCCESS) {
        printf("Failed!!!\n");
        return (1);
    }
    else{
        printf("Success\n");
    }
    
    printf("\nSynch Status:\n");
    
    count = 0;
    
    printf(SPACER "Waiting for synch...");
    fflush(stdout);
    
    while ((status == 0) && (count < 10)) {
       
        /* Retirieve the board's current synch status */
        rv = TPRO_synchStatus(pBoard, &status);
    
        if (rv != TPRO_SUCCESS) {
            printf("Failed retrieving synch status!!!\n");
            return (1);
        }
        
        sleep(1);
        count++;
    }
    
    /* If currently synch'd... */
    if (status) {
        printf("Synch'd\n");

        /* Retrieve the current board time */
        rv = TPRO_getTime(pBoard, &time);
    
        if (rv != TPRO_SUCCESS) {
            printf("Failed retirieving current time!!!\n");
            return (1);
        }
    
        printf(SPACER "Days   : %d\n"
               SPACER "Hours  : %d\n"
               SPACER "Minutes: %d\n"
               SPACER "Seconds: %f\n",
               time.days,
               time.hours,
               time.minutes,
               time.secsDouble);

        printf("\n" SPACER "Disabling Sync...");
        
        syncEnable = 0;
        
        rv = TPRO_synchControl(pBoard, &syncEnable);
        
        sleep(1);
        
        if (rv != TPRO_SUCCESS) {
            printf("Failed\n");
            return (1);
        }
        else {
            printf("Success\n");
        }

        printf(SPACER "Setting new time...");
        
        time.days    = 123;
        time.hours   = 12;
        time.minutes = 34;
        time.seconds = 56;
        
        /* Retrieve the current board time */
        rv = TPRO_setTime(pBoard, &time);
    
        sleep(1);

        if (rv != TPRO_SUCCESS) {
            printf("Failed!!!\n");
            return (1);
        }
        else {
            printf("Success\n");
        }
        
        printf(SPACER "Retrieving new time set...");
        
        /* Retrieve the current board time */
        rv = TPRO_getTime(pBoard, &time);
    
        sleep(1);

        if (rv != TPRO_SUCCESS) {
            printf("Failed!!!\n");
            return (1);
        }
        else {
            printf("Success\n");
        }
    
        printf(SPACER "Days   : %d\n"
               SPACER "Hours  : %d\n"
               SPACER "Minutes: %d\n"
               SPACER "Seconds: %f\n",
               time.days,
               time.hours,
               time.minutes,
               time.secsDouble);        

        printf("\n" SPACER "Enabling Sync...");
        
        syncEnable = 1;
        
        rv = TPRO_synchControl(pBoard, &syncEnable);
        
        sleep(1);
        
        if (rv != TPRO_SUCCESS) {
            printf("Failed\n");
            return (1);
        }
        else {
            printf("Success\n");
        }
        
        /* Retrieve the current board time */
        rv = TPRO_getTime(pBoard, &time);
    
        if (rv != TPRO_SUCCESS) {
            printf("Failed retirieving current time!!!\n");
            return (1);
        }
    
        printf(SPACER "Days   : %d\n"
               SPACER "Hours  : %d\n"
               SPACER "Minutes: %d\n"
               SPACER "Seconds: %f\n",
               time.days,
               time.hours,
               time.minutes,
               time.secsDouble);        
    } /* end - if currently sych'd */
    else {
        printf("Freewheeling!!!\n");
        
        printf(SPACER "Setting new time...");
        
        time.days    = 123;
        time.hours   = 12;
        time.minutes = 34;
        time.seconds = 56;
        
        /* Retrieve the current board time */
        rv = TPRO_setTime(pBoard, &time);
    
        sleep(1);

        if (rv != TPRO_SUCCESS) {
            printf("Failed!!!\n");
            return (1);
        }
        else {
            printf("Success\n");
        }
    }
        
    printf("\nCurrent Time:\n");

    /* Retrieve the current board time */
    rv = TPRO_getTime(pBoard, &time);
    
    if (rv != TPRO_SUCCESS) {
        printf("Failed retirieving current time!!!\n");
        return (1);
    }
    
    printf(SPACER "Days   : %d\n"
           SPACER "Hours  : %d\n"
           SPACER "Minutes: %d\n"
           SPACER "Seconds: %f\n",
           time.days,
           time.hours,
           time.minutes,
           time.secsDouble);

    printf("\nCurrent Date:\n");
    
    /* Retrieve the current Date */
    rv = TPRO_getDate(pBoard, &date);
    
    if (rv != TPRO_SUCCESS) {
        printf("Failed retirieving current date!!!\n");
        return (1);
    }
    
    printf(SPACER "%02d/%02d/%04d\n", date.month, date.day, date.year);

    rv = DisplayPosData(pBoard);
    
    if (rv != TPRO_SUCCESS) {
        return (1);
    }

    printf("\nMatch Time Interrupt:\n");
    
    rv = TestMatch(pBoard);

    if (rv != TPRO_SUCCESS) {
        return (1);
    }

    printf("\nHeartbeat Interrupt:\n");
    
    /* Wait for heartbeat, with 10 second timeout */
    rv = TestHeartbeat(pBoard, 10000);

    if (rv != TPRO_SUCCESS) {
        return (1);
    }

    printf("\nEvent Interrupt:\n");
    
    /* Wait for simulated event */
    
    rv = TestEvent(pBoard);

    if (rv != TPRO_SUCCESS) {
        return (1);
    }
    
    printf("\n");
    
    /* Close board */
    rv = TPRO_close(pBoard);
    
    if (rv != TPRO_SUCCESS) {
        printf ("Could Not Close Board!! [%d]\n", rv);
        return (1);
    }

    return (0);
    
} /* End - main() */

static unsigned char DisplayPosData(TPRO_BoardObj *hnd)
{
    int            rv;
    TPRO_AltObj    altitude;
    TPRO_LatObj    latitude;
    TPRO_LongObj   longitude;
    TPRO_SatObj    satellites;

    printf("\nPosition Data:\n");
    
    /* Retrieve current satellite information */
    rv = TPRO_getSatInfo(hnd, &satellites);
    
    if (rv != TPRO_SUCCESS) {
        printf("Failed retirieving satellite information!!!\n");
        return (rv);
    }
   
    /* Retrieve current altitude */
    rv = TPRO_getAltitude(hnd, &altitude);
    
    if (rv != TPRO_SUCCESS) {
        printf("Failed retirieving altitude!!!\n");
        return (rv);
    }
   
    /* Retrieve current longitude */
    rv = TPRO_getLongitude(hnd, &longitude);
    
    if (rv != TPRO_SUCCESS) {
        printf("Failed retirieving longitude!!!\n");
        return (rv);
    }
   
    /* Retrieve current latitude */
    rv = TPRO_getLatitude(hnd, &latitude);
    
    if (rv != TPRO_SUCCESS) {
        printf("Failed retirieving latitude!!!\n");
        return (rv);
    }
   
    printf(SPACER "Tracking satellites: %d \n", satellites.satsTracked);
    printf(SPACER "Altitude : %f meters\n", altitude.meters);
    printf(SPACER "Latitude : %d degrees\n", latitude.degrees);
    printf(SPACER "           %f minutes\n", latitude.minutes);
    printf(SPACER "Longitude: %d degrees\n", longitude.degrees);
    printf(SPACER "           %f minutes\n", longitude.minutes);

    return (0);
    
} /* End - DisplayPosData() */

/*******************************************************************************
**
*******************************************************************************/
static unsigned char TestEvent(TPRO_BoardObj *pBoard)
{
    TPRO_WaitObj WaitObj;

    int pid;
    unsigned char rv;

    pid = fork();
    
    switch (pid) {

    case -1:
        printf ("Unable to spawn process!\n");
        break;

    /*
    **  Child process
    */
    case 0:
        sleep(5);  /** sleep 5 seconds **/
        printf("Simulated Event\n");
        TPRO_simEvent(pBoard);
        exit(0);

    /*
    **  parent process
    */
    default:
        /*
        **  wait for external time-tag event
        */
        printf(SPACER "Waiting for event...");
        fflush(stdout);
        WaitObj.jiffies = 10 * 1000;
        rv = TPRO_waitEvent(pBoard, &WaitObj);
        
        switch (rv) {

        case TPRO_SUCCESS:
            printf (SPACER "Received Interrupt\n");
            printf (SPACER "  Days: %d\n", WaitObj.days);
            printf (SPACER "  Hours: %d\n", WaitObj.hours);
            printf (SPACER "  Minutes: %d\n", WaitObj.minutes);
            printf (SPACER "  Seconds: %f\n", WaitObj.seconds);
            break;

        case TPRO_TIMEOUT_ERR:
            printf (SPACER "Timeout Waiting for Time-tagged Event!!\n");
            break;

        case TPRO_COMM_ERR:
            printf (SPACER "Error Communicating With Timing Board!!\n");
            break;
        } /* end - switch(rv) */
    } /* end - switch(pid) */

    return (0);

} /* End - TestEvent() */

/*******************************************************************************
**
*******************************************************************************/
static unsigned char TestHeartbeat(TPRO_BoardObj *pBoard, int timeout)
{
    unsigned char rv;
    int ticks;

    ticks = timeout;
    
    printf(SPACER "Waiting....\n");
    
    /* Wait on heartbeat */
    rv = TPRO_waitHeartbeat(pBoard, &ticks);

    switch (rv) {

    case TPRO_SUCCESS: 
        printf(SPACER "Received Heartbeat Interrupt!!\n");
        break;

    case TPRO_TIMEOUT_ERR:
        printf(SPACER "Timeout Receiving Heartbeat!!\n");
        break;

    case TPRO_COMM_ERR:
        printf(SPACER "Error Communicating with Timing Board!!\n");
        break;
    }

    return (rv);
    
} /* End - TestHeartbeat() */


/*******************************************************************************
**
*******************************************************************************/
static unsigned char TestMatch(TPRO_BoardObj *pBoard)
{
    TPRO_TimeObj tproTime;
    TPRO_MatchObj tproMatch;

    unsigned char rv;
    int ticks;

    /* Get time */
    rv = TPRO_getTime(pBoard, &tproTime);
    
    if (rv != TPRO_SUCCESS) {
        printf("Could not retrieve time!! [%d]\n", rv);
        return (1);
    }
    
    printf(SPACER "Current time is: %03d:%02d:%02d:%f\n", tproTime.days,
                                                          tproTime.hours, 
                                                          tproTime.minutes, 
                                                          tproTime.secsDouble);
    /* Set match time */
    tproMatch.matchType = MATCH_TIME_START;
    tproMatch.days      = tproTime.days;
    tproMatch.hours     = tproTime.hours;
    tproMatch.minutes   = tproTime.minutes;
    tproMatch.seconds   = tproTime.secsDouble + 5.000;
    
    if (tproMatch.seconds >= 60.00) {
        tproMatch.seconds -= 60.00;
        tproMatch.minutes++;
    }

    rv = TPRO_setMatchTime(pBoard, &tproMatch);
    
    if (rv != TPRO_SUCCESS) {
        printf("Could not set match time!! [%d]\n", rv);
        return (1);
    }

    printf(SPACER "Match time is:   %03d:%02d:%02d:%f\n", tproMatch.days,
                                                          tproMatch.hours, 
                                                          tproMatch.minutes, 
                                                          tproMatch.seconds);
    /**
    ***  wait for match time
    **/
    printf("\n" SPACER "Waiting for match time....\n");
    ticks = 10 * 1000;  /** 10 seconds **/
    rv = TPRO_waitMatch(pBoard, &ticks);

    switch (rv) {

    case TPRO_SUCCESS: 
        printf(SPACER "Received Match Time Interrupt!!\n");
        break;

    case TPRO_TIMEOUT_ERR:
        printf(SPACER "Timeout Receiving Match Time!!\n");
        break;

    case TPRO_COMM_ERR:
        printf(SPACER "Error Communicating with Timing Board!!\n");
        break;
    }

    return (TPRO_SUCCESS);

} /* End - TestMatch() */

