#include "example.h"

int main (int argc, char *argv[])
{
    char           deviceName[32];
    int            pid;
    int            retVal;
    TPRO_BoardObj *pBoard;
    TPRO_WaitObj   WaitObj;
    unsigned char  rv;

    retVal = EXIT_SUCCESS;
    
    /* If invalid number of arguments... */
    if (argc != 2) 
    {
        printf(" Usage: WaitEvent <device index>\n");
        return EXIT_FAILURE;
    }

    /* Construct the TPRO/TSAT device name */
    sprintf(deviceName, "%s%s%d", DEVICE_ROOT, DEFAULT_DEV, atoi(argv[1]));
    
    /* Open the TPRO/TSAT device */
    rv = TPRO_open(&pBoard, deviceName);
    if (rv != TPRO_SUCCESS) 
    {
        printf(" Error opening '%s': %s (%s:%u)\n", deviceName, 
               tpro_strerror(rv), __FILE__, __LINE__);
        return EXIT_FAILURE;
    }

    /* Wait until this handle is the first user of the device. */
    if (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS) 
    {
        printf(" Waiting to become first user...\n");
        while (TPRO_setPropDelayCorr(pBoard, NULL) != TPRO_SUCCESS);
    }

    pid = fork();
    
    switch (pid) {
    
    case -1:
        printf(" Unable to spawn process!\n");
        break;

    /* Child process */
    case 0:
        sleep (5);  /** sleep 5 seconds **/
        rv = TPRO_simEvent (pBoard);
        if (rv != TPRO_SUCCESS) {
            printf(" Error during simEvent on '%s': %s (%s:%u)\n", 
                   deviceName, tpro_strerror(rv), __FILE__, __LINE__);
            return EXIT_FAILURE; 
        }
        exit (EXIT_SUCCESS);


    /* Parent process */
    default:
        /*
        **  wait for external time-tag event
        */
        printf(" Waiting for event.......");
        fflush (stdout);
        WaitObj.jiffies = 10 * 1000;	/** 10 seconds **/
        rv = TPRO_waitEvent (pBoard, &WaitObj);
        
        switch (rv) {

        case TPRO_SUCCESS:
          printf(" Received Interrupt\n");
          printf("   Days: %d\n", WaitObj.days);
          printf("   Hours: %d\n", WaitObj.hours);
          printf("   Minutes: %d\n", WaitObj.minutes);
          printf("   Seconds: %f\n", WaitObj.seconds);
          break;

        case TPRO_TIMEOUT_ERR:
          printf(" Timeout Waiting for Time-tagged Event!!\n\n");
          retVal = EXIT_FAILURE;
          break;

        case TPRO_COMM_ERR:
          printf(" Error Communicating With Timing Board!!\n\n");
          retVal = EXIT_FAILURE;
          break;
        }
    }

    /* Close the TPRO/TSAT device */
    rv = TPRO_close(pBoard);
    if (rv != TPRO_SUCCESS) {
        printf(" Error closing '%s': %s\n", deviceName, tpro_strerror(rv));
        retVal = EXIT_FAILURE;
    }

    return (retVal);

} /* End - main() */
