#include "example.h"

#include <string.h>
#include <time.h>

#define SYNC_TIMEOUT 30

static unsigned char TEST_DisplayDateTime (TPRO_BoardObj *boardPtr);
static unsigned char TEST_DisableSync     (TPRO_BoardObj *boardPtr);
static unsigned char TEST_EnableSync      (TPRO_BoardObj *boardPtr);
static unsigned char TEST_Tsat            (TPRO_BoardObj *boardPtr);


int main (int argc, char *argv[])
{
    char           deviceName[32];
    TPRO_BoardObj *pBoard;
    TPRO_TimeObj   setTime;
    struct tm      pcTime;
    time_t    timet;
    unsigned short year;
    char firmware[4];
    char driver[7];
    unsigned char status;
    unsigned char rv;
    unsigned int count;
    /* TODO: buffer overrun potential */
    char type[32];
    unsigned char origSync;
    // If given an incorrect number of arguments, OR invalid device...
    if ((argc != 3) || 
        ((strcmp(argv[1], "tpro") != 0) &&
         (strcmp(argv[1], "tsat") != 0)))
    {
        printf ("Usage: testtpro <device type> <device index>\n"
                "Device Types:\n"
                "tpro - \n"
                "tsat - \n");
        return (1);
    }
    
    strcpy(type, argv[1]);
    
    sprintf(deviceName, "%s%s%d", DEVICE_ROOT, DEFAULT_DEV, atoi(argv[2]));
    
    // If unable to open the specified device index...
    if ((rv = TPRO_open(&pBoard, deviceName)) != TPRO_SUCCESS) 
    {
        printf ("FAILED: Could Not Open '%s'!! [%d]\n", deviceName, rv);
        return (1);
    }

    //
    // Retrieve and display the TPRO/TSAT PCI firware revision.
    //
    printf("\n Firmware revision: ");
    
    if (TPRO_getFirmware(pBoard, firmware) != TPRO_SUCCESS)
    {
        printf("Failed!!!\n");
        (void)TPRO_close(pBoard);
        return (1);
    }
    
    printf("%02X-%02X-%02X-%02X\n",
           firmware[0], firmware[1], firmware[2], firmware[3]);
    
    //
    // Retrieve and display the TPRO/TSAT PCI driver revision.
    //
    printf(" Driver revison:    ");

    if (rv = TPRO_getDriver(pBoard, driver), rv != TPRO_SUCCESS) 
    {
        printf ("Failed!!!\n");
        (void)TPRO_close(pBoard);
        return (1);
    }
    
    printf ("%s\n\n", driver);
    
    //
    // Retrieve and display the TPRO/TSAT's current sync status
    //
    rv = TPRO_synchStatus(pBoard, &origSync);
    
    printf(" Retrieving synchronization status...");
    
    if (rv != TPRO_SUCCESS) 
    {
        printf ("Failed!!!\n");
        (void)TPRO_close(pBoard);
        return (1);
    }
    
    // If the board is current'y sync'd...
    if (origSync) 
    {
        //
        // Force the board into freewheeling and check for unsync'd state
        //
        printf("Sync'd\n\n");
        
        //
        // Check the time
        //
        if (TEST_DisplayDateTime(pBoard) != TPRO_SUCCESS)
        {
            (void)TPRO_close(pBoard);
            return (1);
        }

        printf(" Forcing Freewheeling...");
                
        // If unable to force the freewheeling mode...
        if (TEST_DisableSync(pBoard) != TPRO_SUCCESS)
        {
            printf("Failed\n\n");
            (void)TPRO_close(pBoard);
            return (1);
        }
        
        printf("Successful\n\n"
               " Checking sync...");
        fflush(stdout);
        
        // Give board time to unsync
        sleep (10);
        
        //
        // Retrieve and display the new sync state
        //
        rv = TPRO_synchStatus(pBoard, &status);
        
        if (rv != TPRO_SUCCESS) 
        {
            printf("Failed!!!\n");
            (void)TPRO_close(pBoard);
            return (1);
        }
        else if (status)
        {
            printf("Failed: TPRO still sync'd!!!\n\n");
            (void)TPRO_close(pBoard);
            return (1);
        }
        
        printf("No longer sync'd\n\n");

        printf(" Reenabling sync...");
                
        // If unable to force the freewheeling mode...
        if (TEST_EnableSync(pBoard) != TPRO_SUCCESS)
        {
            printf("Failed\n\n");
            (void)TPRO_close(pBoard);
            return (1);
        }
    }
    else 
    {
        printf ("Not sync'd\n\n");
        
        //
        // Check the time
        //
        if (TEST_DisplayDateTime(pBoard) != TPRO_SUCCESS)
        {
            (void)TPRO_close(pBoard);
            return (1);
        }
    }

    //
    // Reset the firmware of the TPRO/TSAT PCI board
    //
    printf(" Resetting the board...");
    fflush(stdout);

    if (TPRO_resetFirmware(pBoard) != TPRO_SUCCESS)
    {
        printf("Failed!!!\n");
        (void)TPRO_close(pBoard);
        return (1);
    }

    // Documentation says to give at least 8 seconds after a reset
    sleep(10);

    printf("Successful\n\n");

    // If the PCI board originally didn't have sync...
    if (!origSync)
    {
        //
        // Display the time the board currently has, it should have been reset
        //
        if (TEST_DisplayDateTime(pBoard) != TPRO_SUCCESS)
        {
            (void)TPRO_close(pBoard);
            return (1);
        }

        // Get today's date and time
        (void)time(&timet);
        (void)gmtime_r(&timet, &pcTime);

        //
        // Set the current time in the card 
        //
        printf(" Setting Time/Date of TPRO-PCI device...");

        setTime.seconds = (unsigned char)pcTime.tm_sec;
        setTime.minutes = (unsigned char)pcTime.tm_min;
        setTime.hours   = (unsigned char)pcTime.tm_hour;
        setTime.days    = (unsigned short)pcTime.tm_yday + 1;

        // If error while attempting to set the TPRO's time...
        if (TPRO_setTime(pBoard, &setTime) != TPRO_SUCCESS)
        {
            printf("Failed!!!\n");
            (void)TPRO_close(pBoard);
            return (1);
        }

        //
        // Set the current year in the card
        //
        printf("Successful\n\n"
               " Setting Year of TPRO-PCI device...");

        year = (unsigned short)pcTime.tm_year + 1900;

        if (TPRO_setYear(pBoard, &year) != TPRO_SUCCESS)
        {
            printf("Failed!!!\n");
            (void)TPRO_close(pBoard);
            return (1);
        }
        printf("Successful\n\n");

        // Allow for new time to take effect
        sleep(1);
    }
    else
    {
        printf(" Checking sync...");
               
        status = 0;
        count  = 0;
    
        while ((!status) && (count < SYNC_TIMEOUT))
        {
            rv = TPRO_synchStatus(pBoard, &status);
        
            if (rv != TPRO_SUCCESS) 
            {
                printf("Failed!!!\n");
                (void)TPRO_close(pBoard);
                return (1);
            }
        
            sleep(1);
        
            count++;
        }
        
        if (status)
        {
            printf("Sync re-established!!!\n\n");
        }
        else
        {
            printf("Sync couldn't be re-established!!!\n\n");
        }
    }
    
    //
    // Check the newly set time
    //
    if (TEST_DisplayDateTime(pBoard) != TPRO_SUCCESS)
    {
        (void)TPRO_close(pBoard);
        return (1);
    }

    if (strcmp(type, "tsat") == 0)
    {
        TEST_Tsat(pBoard);    
    }
    
    // The device couldn't be closed...
    if (TPRO_close(pBoard) != TPRO_SUCCESS) 
    {
        printf ("Could Not Close Board!! [%d]\n", rv);
        return (1);
    }

    return (0);

} // End - main()

/*******************************************************************************
**
** Function: TEST_DisplayDateTime()
**
** Description: Display Date/Time retrieved from TPRO/TSAT PCI board.
**
** Parameters:
**     IN: *boardPtr - Pointer to device we are currently testing
**
**     RETURNS: TPRO_SUCCESS - Success
**
*******************************************************************************/
static unsigned char TEST_DisplayDateTime(TPRO_BoardObj *boardPtr)
{
    TPRO_DateObj  tproDate;
    TPRO_TimeObj  tproTime;
    unsigned char rv;
    
    printf(" Getting Time/Date from TPRO-PCI device...");

    // Retrieve the time from the unit
    rv = TPRO_getTime(boardPtr, &tproTime);
    
    // If error while retrieving the time...
    if (rv != TPRO_SUCCESS) 
    {
        printf("Failed!!!\n");
        return (rv);
    }
    
    rv = TPRO_getDate(boardPtr, &tproDate);
    
    if (rv != TPRO_SUCCESS) 
    {
        printf("Failed!!!\n");
        return (rv);    
    }
    
    printf("Successful\n"
           "  Days:        %d\n"
           "  Hours:       %d\n"
           "  Minutes:     %d\n"
           "  Seconds:     %f\n"
           "  Date:        %02d/%02d/%04d\n\n", 
           tproTime.days, tproTime.hours, tproTime.minutes, tproTime.secsDouble,
           tproDate.month, tproDate.day, tproDate.year);


    return (TPRO_SUCCESS);
    
} // End - TEST_DisplayDateTime()

/*******************************************************************************
**
** Function: TEST_DisableSync()
**
** Description: Forces the TPRO/TSAT PCI board into freewheeling.
**
** Parameters:
**     IN: *boardPtr - Pointer to device we are currently testing
**
**     RETURNS: TPRO_SUCCESS - Success
**
*******************************************************************************/
static unsigned char TEST_DisableSync(TPRO_BoardObj *boardPtr)
{
    unsigned char force = 0x00;        
            
    return (TPRO_synchControl(boardPtr, &force));

} // End - TEST_DisableSync()

/*******************************************************************************
**
** Function: TEST_EnableSync()
**
** Description: Enables the TPRO/TSAT PCI board to sync with its reference.
**              This disables the forced freewheeling mode.
**
** Parameters:
**     IN: *boardPtr - Pointer to device we are currently testing
**
**     RETURNS: TPRO_SUCCESS - Success
**
*******************************************************************************/
static unsigned char TEST_EnableSync(TPRO_BoardObj *boardPtr)
{
    unsigned char force = 0x01;        
            
    return (TPRO_synchControl(boardPtr, &force));

} // End - TEST_EnableSync()

/*******************************************************************************
**
** Function: TEST_Tsat()
**
** Description: Exercise additional functionality of a TSAT-PCI unit.
**
** Parameters:
**     IN: *boardPtr - Pointer to device we are currently testing
**
**     RETURNS: TPRO_SUCCESS - Success
**
*******************************************************************************/
static unsigned char TEST_Tsat(TPRO_BoardObj *boardPtr)
{
    TPRO_LatObj   longitude;
    TPRO_LongObj  latitude;
    TPRO_SatObj   satInfoPtr;
    unsigned char rv;
    
    if ((rv = TPRO_getSatInfo(boardPtr, &satInfoPtr)) != TPRO_SUCCESS)
    {
        printf("Error: Couldn't retrieve satellite information\n\n");
        return (rv);
    }
    
    if ((rv = TPRO_getLatitude(boardPtr, &latitude)) != TPRO_SUCCESS)
    {
        printf("Error: Couldn't retrieve latitude\n\n");
        return (rv);
    }
    
    if ((rv = TPRO_getLongitude(boardPtr, &longitude)) != TPRO_SUCCESS)
    {
        printf("Error: Couldn't retrieve longitude\n\n");
        return (rv);
    }
    
    printf(" Number of satellites being tracked: %d\n"
           " Longitude:\n"
           "    Degrees: %d\n"
           "    Minutes: %f\n"
           " Latitude:\n"
           "    Degrees: %d\n"
           "    Minutes: %f\n\n",
           satInfoPtr.satsTracked,
           longitude.degrees, longitude.minutes,
           latitude.degrees, latitude.minutes);
    
    return (TPRO_SUCCESS);
    
} // End - TEST_Tsat()


